NEW;                       /* File: LINTEST.PRG */
cls;
/*------------------------------- REFERENCES ---------------------------------*

We performing a linearity test against a STAR alternative following: Granger
and Tersvirta, "Modelling Nonlinear Economic Relationships", Advanced
Texts in Economics (1993). The test referred to as "Standard non-linearity
test" is described at p 65-66 with constant residual variance. The test
referred to as "Robustified non-linearity test" is described at p 69-70 and
is robust against unspecified heteroskedasticity.

First version 960416

 *----------------------------------------------------------------------------*/
/*--------------------------- USER SPECIFICATION -----------------------------*/
/*----------------------------------------------------------------------------*/


               /*------------ GENERAL PARAMETERS ------------*/
dmax=10;        /* Maximum delay parameter of the STAR model under the
                   alternative. */

pmax=8;        /* Maximum and minimum laglength of the linear and nonlinear */
pmin=4;         /* AR parts of the STAR model under the alternative. */

kmax=3;         /* Maximum order k of the STAR model under the alternative. */

choice=0;       /* The choice of what to calculate:
                   0  Calculate only the standard test.
                   1  Calculate only the robustified test.
                   2  Calculate both. */

forecast= 0;		@ save last 0 observations for forcasts @ 
diff 	= 0;		@ first diff: 1 if used, 0 otherwise @
selday	= 0;		@ dummies: 1 if present, 0 otherwise @

Series= "sp500";                                 /* Chose what to analyse. */

	       /*--------------- FILE PARAMETERS ---------------*/

baseDIR= "c:\\wingauss\\mydir";
savefile= baseDIR $+ "\\" $+ series $+ "_lintest.txt";


{input,dum,dummies}=getData2(diff,savefile,Series,selday,forecast);



T=rows(input);
	    
          /*------------ Initialise the savefile ----------*/


    result0f=ones(dmax,pmax-pmin+1);
    result0c=ones(dmax,pmax-pmin+1);
    result1=ones(dmax,pmax-pmin+1);

/*-------------- Calculate test for combinations of d and p. ------------*/


    d=1;
    et=hsec;
    do until d > dmax;
        p=pmin;
        do until p > pmax;
            z=test(input,kmax,p,d,choice);
            IF choice==0 or choice==2;
                result0f[d,p-pmin+1]=cdffc(z[1,1],z[1,2],z[1,3]);
                result0c[d,p-pmin+1]=cdfchic(z[2,1],z[2,2]);
            ENDIF;
            IF choice==1 or choice==2;
                result1[d,p-pmin+1]=cdfchic(z[3,1],z[3,2]);
            ENDIF;
            clear z;
            p=p+1;
        endo;

        d=d+1;
     endo;

/*--------------------- Print result(s) to file. ------------------------*/

    print " ";
    output file=^savefile on;
    format /rd 1,2;
    print "Total calculation time: " (hsec-et)/6000 "minutes.";

    IF choice==0 or choice==2;
        result0f=seqa(pmin,1,pmax-pmin+1)'|result0f;
        result0f=seqa(0,1,dmax+1)~result0f;
        print " ";
        print "The Standard non-linearity test. (F version)";
        out(result0f);

        result0c=seqa(pmin,1,pmax-pmin+1)'|result0c;
        result0c=seqa(0,1,dmax+1)~result0c;
        print " ";
        print "The Standard non-linearity test. (Chi-square version)";
        out(result0c);
    ENDIF;

    IF choice==1 or choice==2;
        result1=seqa(pmin,1,pmax-pmin+1)'|result1;
        result1=seqa(0,1,dmax+1)~result1;
        print " ";
        print "The Robustified non-linearity test. (Chi-square version)";
        out(result1);
    ENDIF;

    print " ";
    print " ";
output off;

END;  /******************* END OF LINTEST.PGM *************************/
/**************************************************************************
 * Procedure test.                                                        *
 *                                                                        *
 * This procedure calculate linearity tests with a STAR model as          *
 * alternative. There are two version of the test, one standard and one   *
 * robustified.                                                           *
 *                                                                        *
 * Arguments of the procedure are:                                        *
 *                                                                        *
 * input       A row vector with the actual timeserie. Where the first    *
 *             observation is the last (in time).                         *
 * kmax        The constant that specify the polynomia without the        *
 *             restriction(s).                                            *
 * p           The maximum lag of the AR(p) polynomia.                    *
 * d           The lag of the nonlinearity in the STAR model.             *
 * choice      If  0  Calculate only the standard test.                   *
 *                 1  Calculate only the robustified test.                *
 *                 2  Calculate both.                                     *
 *                                                                        *
 * The procedure returns a (3,3)-matrix with significant information      *
 * about the linearity test(s). The first two rows are associated with    *
 * the standard version and elements in the FIRST row are in order:       *
 * F-statistic, Nominator DF (degrees of freedom) and Dominator DF.       *
 * The SECOND row is the chi-square version of the standard test with the *
 * teststatistic and degrees of freedom. The THIRD row is associated with *
 * the robustified version and the elements are in order: Chi-square      *
 * statistic and DF.                                                      *
 *************************************************************************/

PROC(1)=test(input,kmax,p,d,choice);

LOCAL Y,X,M,T,i,s,j,AR,Ydelay,respond,SSR0,SSR1,F,resid,temp,H,ONE,U,MH,UMH;

T=rows(input);
respond=ones(3,3);

IF p>=d;

    Y=trimr(input,p,0);        /* Generate the Y vector. */
    Y=(Y-meanc(Y))/stdc(Y);    /* Standardize Y. */

    AR=trimr(input,p-1,1);     /* Creating the AR matrix, used as a */
    i=2;                       /* part when building the M matrix. */
    do until i > p;
        AR=AR~trimr(input,p-i,i);
        i=i+1;
    endo;

    M=ones(T-p,1)~AR;           /* Creating the M matrix which is used */
    Ydelay=trimr(input,p-d,d);  /* when calculating SSR1. */
    s=1;
    do until s > kmax;
        M=M~AR.*Ydelay^s;
        s=s+1;
    endo;

    i=1;                        /* Standardize M (and indirectly X) */
    do until i > cols(M);
        if meanc(M[.,i])==1;
            M[.,i]=M[.,i];
        else;
            M[.,i]=(M[.,i]-meanc(M[.,i]))/stdc(M[.,i]);
        endif;
        i=i+1;
    endo;

    X=M[.,1:p+1];               /* Generating the X matrix (using the first
                                   columns of the M matrix) which is used
                                   when calcualting SSR0. */

/*-------------- Calculate the Standard non-linearity test. -------------*/

    IF choice==0 or choice==2;

    SSR0=Y'Y-Y'X*invpd(X'X)*X'Y;
    SSR1=Y'Y-Y'M*invpd(M'M)*M'Y;

    /* Calculating the F-test statistic */
    F=((SSR0-SSR1)/(p*kmax))/(SSR1/(T-p-(p*(kmax+1)+1)));

    respond[1,1]=F;                        /* Test statistic. */
    respond[1,2]=p*kmax;                   /* Numerator df. */
    respond[1,3]=T-p-(p*(kmax+1)+1);       /* Denominator df. */

    /* Calculate the Chi-square statistic */
    respond[2,1]=(SSR0-SSR1)/(SSR0/(T-p)); /* Test statistic. */
    respond[2,2]=p*kmax;                   /* Degrees of freedom. */

    ENDIF;

/*------------- Calculate the Robustified non-linearity test ------------*/

    IF choice==1 or choice==2;

    /* Step(i) */
    resid=Y-X*invpd(X'X)*X'Y;
    temp=zeros(rows(resid),rows(resid));

    /* Step(ii) */
    H=M[.,p+2:cols(M)];
    MH=H-X*invpd(X'X)*X'H;

    /* Step(iii) */
    U=diagrv(temp,resid);
    UMH=U*MH;
    ONE=ones(rows(UMH),1);
    respond[3,1]=ONE'UMH*invpd(UMH'UMH)*UMH'ONE;  /* Test statistic. */
    respond[3,2]=p*kmax;                          /* Degrees of freedom. */

    ENDIF;

ELSE; /*------------------------- When p<d ------------------------------*/

    Y=trimr(input,d,0);        /* Generate the Y vector. */
    Y=(Y-meanc(Y))/stdc(Y);    /* Standardize Y. */

    AR=trimr(input,d-1,1);     /* Creating the AR matrix, used as a */
    i=2;                       /* part when building the M matrix. */
    do until i > p;
        AR=AR~trimr(input,d-i,i);
        i=i+1;
    endo;
    Ydelay=trimr(input,0,d);
    AR=AR~Ydelay;

    M=ones(T-d,1)~AR;           /* Creating the M matrix which is used */
    s=1;                        /* when calculating SSR1. */
    do until s > kmax;
        M=M~AR.*Ydelay^s;
        s=s+1;
    endo;

    i=1;                        /* Standardize M (and indirectly X) */
    do until i > cols(M);
        if meanc(M[.,i])==1;
            M[.,i]=M[.,i];
        else;
            M[.,i]=(M[.,i]-meanc(M[.,i]))/stdc(M[.,i]);
        endif;
        i=i+1;
    endo;

    X=M[.,1:(p+1)+1];           /* Generating the X matrix (using the first
                                   columns of the M matrix) which is used
                                   when calcualting SSR0. */

/*---------------- Calculate the Standard non-linearity test ------------*/

    IF choice==0 or choice==2;

    SSR0=Y'Y-Y'X*inv(X'X)*X'Y;
    SSR1=Y'Y-Y'M*inv(M'M)*M'Y;

    /* Calculating the F-test statistic */
    F=((SSR0-SSR1)/((p+1)*kmax))/(SSR1/(T-d-((p+1)*(kmax+1)+1)));

    respond[1,1]=F;                           /* Test statistic. */
    respond[1,2]=(p+1)*kmax;                  /* Numerator df. */
    respond[1,3]=T-d-((p+1)*(kmax+1)+1);      /* Denominator df. */

    /* Calculate the Chi-square statistic */
    respond[2,1]=(SSR0-SSR1)/(SSR0/(T-d));    /* Test statistic. */
    respond[2,2]=(p+1)*kmax;                  /* Degrees of freedom. */

    ENDIF;

/*------------- Calculate the Robustified non-linearity test ------------*/

    IF choice==1 or choice==2;

    /* Step(i) */
    resid=Y-X*invpd(X'X)*X'Y;
    temp=zeros(rows(resid),rows(resid));

    /* Step(ii) */
    H=M[.,(p+1)+2:cols(M)];
    MH=H-X*invpd(X'X)*X'H;

    /* Step(iii) */
    U=diagrv(temp,resid);
    UMH=U*MH;
    ONE=ones(rows(UMH),1);
    respond[3,1]=ONE'UMH*invpd(UMH'UMH)*UMH'ONE;  /* Test statistic. */
    respond[3,2]=(p+1)*kmax;                      /* Degrees of freedom. */
    ENDIF;

ENDIF;

RETP(respond);
ENDP;  /******************* END OF PROC LINTEST *************************/

/**************************************************************************
 * Procedure out.                                                         *
 *                                                                        *
 * This procedure print the result matrix.                                *
 *                                                                        *
 * Argument of the procedure is:                                          *
 *                                                                        *
 * result      The matrix to be printed                                   *
 *                                                                        *
 * No returns from the procedure.                                         *
 *************************************************************************/

PROC(0)=out(result);

LOCAL i;


format /rd 5,0; print "      p=" result[1,2];;
format /rd 12,0; result[1,3:cols(result)];

print "d=";
i=2;
do until i > rows(result);
    format /rd 2,0; print result[i,1] "   ";;
    format /res 12,3; print result[i,2:cols(result)];
    i=i+1;
endo;

ENDP;  /******************** END OF PROC OUT **************************/

