NEW;                       /* File: STARGRID.PRG */
cls;

/*----------------------------------------------------------------------------*/
/*--------------------------- USER SPECIFICATION -----------------------------*/
/*----------------------------------------------------------------------------*/

forecast= 0;		@ save last 0 observations for forcasts @ 
diff 	= 0;		@ first diff: 1 if used, 0 otherwise @
selday 	= 0;		@ dummies: 1 if present, 0 otherwise @

	       /*------------ GENERAL PARAMETERS ------------*/

d=1;                       /* Delay parameter for STAR model. */
k=1;                       /* Order of the STAR model. */

/* Define the linear and non-linear AR parts of the model, if 0 is present
   it indicate a constant term. All parameters should be in successive order.
   "plin" is the row vector that contains the lags apriori different from 0
   for the linear AR part. "pnonlin" is the same but for the non-linear AR
   part. */

plin=0|seqa(1,1,7);
pnonlin=plin;

Series= "sp500";	/* Chose asset to analyse. */


eps=0.02 ;     /* Used to check when the STAR model reduces to
			      a simple linear model */

/* Initial restrictions. If there is no restrictions initially, use an empty
   vector. In the case when restriction(s) is/are present, stack the lag(s)
   in a row vector. */

restr={};

	       /*--------------- FILE PARAMETERS ---------------*/

baseDIR= "c:\\wingauss\\mydir";
savefile= baseDIR $+ "\\" $+ series $+ "_grid.txt";


{input,dum,dummies}=getData2(diff,savefile,Series,selday,forecast);

	       /*-------------- GRID PARAMETERS ---------------*/

  
ginfo=0.5~10~30;   /* Start, stop and number of steps for Gamma.
			         The distance between sucessive q are konstant
			         after ln-transformation. */

/* Each of the k rows in cinfo contains Start, stop and step length for the
   constant(s). Here can a lot of calculations be eliminated if we chose a
   symmetric grid. In that case we iterate grid only for c1<=c2<=c3...<=ck. */

cinfo={};
inigrid=(-0.1)~0.1~40;
i=1;
do until i > k;
    cinfo=cinfo|inigrid;
    i=i+1;
endo;


/*----------------------------------------------------------------------------*/
/*------------------------ END OF USER SPECIFICATION -------------------------*/
/*----------------------------------------------------------------------------*/


   /*------------ Data transformation and some data description ----------*/


plin=sortc(plin,1);              /* Precautionary sorting. */
pnonlin=sortc(pnonlin,1);
restr=sortc(restr,1);



pmax=plin[rows(plin)]|pnonlin[rows(pnonlin)];  /* Finding the maximum lag */
p=maxc(pmax);


/*---------------- Calculate some parts of the STAR model -------------------*/

	 /*------Adjusting the length of matrix and vectors -------*/

IF scalmiss(dummies)==0;
    dummies=dummies[1+p:rows(input),.];
ENDIF;

Y=input[1+p:rows(input)];
Ydelay=input[1+p-d:rows(input)-d];
dstd=stdc(Ydelay);                        /* Variance of delay vector */


		    /*------- Linear AR matrix ----------*/
      /* Used when constructing the linear AR part of the STAR model. */
   
IF plin[1]==0;     /* This condition checks whether a constant is present */
    ARL=ones(rows(input)-p,1);
    i=2;
ELSE;
    ARL=input[1+p-plin[1]:rows(input)-plin[1]];
    i=2;
ENDIF;

do until i > rows(plin);
    ARL=ARL~input[1+p-plin[i]:rows(input)-plin[i]];
    i=i+1;
endo;

orgARL=ARL;

      /*------- Change ARL if restriction(s) is/are present. ---------*/

IF scalmiss(restr)==0;           /* If true, there is/are restriction(s). */
    indrest=indnv(restr,plin);   /* Create a vectorindex for the parameter(s)*/ 
				 /* that are restricted in plin. */
    ARLtemp={};
    j=1;
    do until j > rows(plin); 
	if scalmiss(indexcat(indrest,j))==1; /* If true, there is no restr */
					     /* for index j in plin. */
	    ARLtemp=ARLtemp~ARL[.,j];
	endif;
	j=j+1;
    endo;
    ARL=ARLtemp; 
ENDIF;


ARDL=dummies~ARL;    /* Matrix of Dummy matrix and AR matrix */


		 /*-------- Non-linear AR matrix ---------*/
     /* Used when constructing the non-linear AR part of the STAR model */

IF pnonlin[1]==0;     /* This condition checks whether a constant is present */
    ARN=ones(rows(input)-p,1);
    i=2;
ELSE;
    ARN=input[1+p-pnonlin[1]:rows(input)-pnonlin[1]];
    i=2;
ENDIF;

do until i > rows(pnonlin);
    ARN=ARN~input[1+p-pnonlin[i]:rows(input)-pnonlin[i]];
    i=i+1;
endo;

orgARN=ARN;



                  /*---------- Calculate a grid ------------*/

m=grid(ARL,ARN,dummies,Y,Ydelay,cinfo,ginfo,eps,k,restr,pnonlin);



if k==1;
    plotm1(m,ginfo,cinfo);  
endif;

if k==2;
    plotm2(m,ginfo,cinfo);
endif;


name="SSR"|"Gamma"|(0$+"C"$+ftocv(seqa(1,1,k),1,0));
allmin=minc(m);
indmin=indexcat(m[.,1],allmin[1]);
start=m[indmin,.];

/* Check whether the cinfo is symmetric in the sense that all rows are equal */
i=1;
sym=1;                                /* Assume symmetric */
do until i > k;
    if not cinfo[1,.]==cinfo[i,.];
	sym=0;                            /* If not symmetric */
    endif;
    i=i+1;
endo;

output file=^savefile on; 

print " ";
print " ";

format /rd 1,0;
if k==1;
    print "Resolution of the grid (Combinations): ";; 
    sizeM=ginfo[3]*cinfo[1,3];
elseif sym==1;
    print "Resolution of the symmetric grid (Combinations): ";; 
    sizeM=ginfo[3];
    i=1;
    do until i > k;
        sizeM=sizeM*(cinfo[i,3]-1+i)/i;
        i=i+1;
    endo;
else;
    print "Resolution of the nonsymmetric grid (Combinations): ";; 
    sizeM=ginfo[3]; 
    i=1;
    do until i > k;
        sizeM=sizeM*cinfo[i,3];
        i=i+1;
    endo;   
endif;
print sizeM;

print " ";

print "Parameter       Start      Stop       Number of steps";
format /ld 10,3;  print "Gamma      ";; 
format /rd 10,3;  print ginfo[1,1] ginfo[1,2];; 
format /rd 10,0;  print "   " ginfo[1,3];
i=1;
do until i > k;
    format /ld 10,3; print $name[i+2];;
    format /rd 10,3; print cinfo[i,1] cinfo[i,2];;
    format /rd 10,0; print "   " cinfo[i,3];
    i=i+1;
endo;

print " ";
print " ";
print "Startvalues of the parameters: ";
print " ";
format /m1 /ld 8,6;
i=1;
do until i > rows(name);
    print $name[i] ":" ;;
    print start[i];
    i=i+1;
endo;
output off;

end;



/*----------------------------------------------------------------------------*/
/*-------------------------------- PROCEDURES --------------------------------*/
/*----------------------------------------------------------------------------*/



/*----------------------------------------------------------*/
proc(0)=plotm1(m,ginfo,cinfo);
LOCAL numg,numc1,x,y,z,i,j,txt;


library pgraph;
graphset;


 numg=ginfo[1,3];
numc1=cinfo[1,3];

y=m[1:numc1,3];
z=zeros(numc1,numg);
x=zeros(1,numg);

i=1;
do until i > numg;
    x[1,i]=m[(i-1)*numc1+1,2];
    j=1;
    do until j > numc1;
        z[j,i]=m[(i-1)*numc1+j,1];
        j=j+1;
    endo;
    i=i+1;
endo;

pqgwin auto;
begwind;
makewind(9,6.855,0,0,0);
title("Contour plot");
xlabel("gamma-parameter");
ylabel("c1-parameter");
zlabel("Square sum of residuals");
 contour(x[1,2:numg],y[2:numc1,1],z[2:numc1,2:numg]); 
endwind;

begwind;
makewind(9,6.855,0,0,0);
title("Surface plot");
xlabel("gamma-parameter");
ylabel("c1-parameter");
zlabel("Square sum of residuals");
 surface(x,y,-z); 
endwind;

endp;
/*------------------------------------------------*/

proc(0)=plotm2(m,ginfo,cinfo);
LOCAL numg,numc1,numc2,x,y,z,i,subMsize,sym,cstep,cval,j,l,k,txt,gval,gstep,step;
LOCAL minordiv;

library pgraph;
graphset;
pqgwin manual;


 numg=ginfo[1,3];
numc1=cinfo[1,3];
numc2=cinfo[2,3];

x=zeros(1,numc2);
y=zeros(numc1,1);
z=zeros(numc1,numc2);

cstep=(cinfo[.,2]-cinfo[.,1])./(cinfo[.,3]-1);

cval=cinfo[1,1];
y[1,1]=cval;
i=2;
do until i > numc1;
    cval=cval+cstep[1,1];
    y[i,1]=cval;
    i=i+1;
endo;

cval=cinfo[2,1];
x[1,1]=cval;
i=2;
do until i > numc2;
    cval=cval+cstep[2,1];
    x[1,i]=cval;
    i=i+1;
endo;

/* Check whether the cinfo is symmetric in the sense that all rows are equal */
i=1;
k=rows(cinfo);
sym=1;                                /* Assume symmetric */
do until i > k;
    if not cinfo[1,.]==cinfo[i,.];
	sym=0;                            /* If not symmetric */
    endif;
    i=i+1;
endo;



l=1;
do until l > numg;

    subMsize=rows(M)/numg;
    IF sym==1;
        k=1;
        i=1;
        do until i > numc1;
            j=i;
            do until j > numc2;
                z[i,j]=M[(l-1)*subMsize+k,1];
                z[j,i]=z[i,j];
                k=k+1;
                j=j+1;
            endo;
            i=i+1;
        endo;
    ELSE;
        i=1;
        do until i > numc1;
            j=1;
            do until j > numc2;
                z[i,j]=M[(l-1)*subMsize+(i-1)*numc1+j,1];
                j=j+1;
            endo;
            i=i+1;
        endo;
    ENDIF;

    gstep=(ginfo[.,2]-ginfo[.,1])./(ginfo[.,3]-1);
    gval=ginfo[1,1]+(l-1)*gstep;
    txt=0$+"g="$+ftocv(gval,2,2);  

    begwind;

    step=(maxc(m[.,1])-minc(m[.,1]))/5;
    minordiv=(maxc(m[.,1])-minc(m[.,1]))/10;
    ztics(-maxc(m[.,1]),-minc(m[.,1]),step,minordiv);

    makewind(9,6.855,0,0,0);
    title(txt);
    xlabel("c2-parameter");
    ylabel("c1-parameter");
    zlabel("Square sum of residuals");
     surface(x,y,-z); 
    endwind;
/*
    pqgwin auto;
    begwind;
    makewind(9,6.855,0,0,0);
    title(txt);
    xlabel("c2-parameter");
    ylabel("c1-parameter");
    zlabel("Square sum of residuals");
     contour(x[1,2:numc2],y[2:numc1,1],z[2:numc1,2:numc2]); 
    endwind;
*/   

    l=l+1;
endo; 
endp;

/*------------------------------------------------*/



/**************************************************************************
 * Procedure GRID                                                         *
 *                                                                        *
 * This procedure calculate a grid for a STAR model.                      *
 *                                                                        *
 * ARGUMENTS to the function are:                                         *
 *                                                                        *
 * ARL         The linear AR part of the STAR model.                      *
 * ARN         The non-linear AR part of the STAR model.                  *
 * dummies      Dummyvariables used if data is daily.                      *
 * Y, Ydelay   Observation vectors.                                       *
 * cinfo       Contains start, stop and step-values for Constant(s).      *
 * ginfo       Contains start, stop and step-values for Gamma.            *
 * eps         Value when STAR model reduces to a linear model.           *
 * k           Order of STAR model.                                       *
 * restr       The restriction vector.                                    *
 * pnonlin     Apriori for nonlinear AR polynomia.                        *
 *                                                                        *
 * The procedure RETURNS a matrix where each row consist of               *
 * SSR, g, c1,..., ck.                                                    *
 *                                                                        *
 *************************************************************************/

PROC(1)=grid(ARL,ARN,dummies,Y,Ydelay,cinfo,ginfo,eps,k,restr,pnonlin);
LOCAL j,i,M,g,gstep,c1,c2,c3,c4,cvec,cstep,temp,ind,indrest;
LOCAL sizeM,ARDL,ARR,ssr1,ssr2,sym,txt,cx,cy,X;

/* Check whether the cinfo is symmetric in the sense that all rows are equal */
i=1;
sym=1;                                /* Assume symmetric */
do until i > k;
    if not cinfo[1,.]==cinfo[i,.];
	sym=0;                            /* If not symmetric */
    endif;
    i=i+1;
endo;


/* Create a vectorindex for the restricted parameters in pnonlin.
   Calculate the SSR of the linear model used when the linear approximation
   of the STAR model reduces to a simple linear model */

ARDL=dummies~ARL;

IF scalmiss(restr)==0;
    indrest=indnv(restr,pnonlin);
    ARR=-ARN[.,indrest];
    X=ARDL~ARR;
    SSR1=Y'Y-Y'X*invpd(X'X)*X'Y;

ELSE;
    indrest={};
    X=ARDL;
    SSR1=Y'Y-Y'X*invpd(X'X)*X'Y;    
ENDIF;
X=ARDL;
SSR2=Y'Y-Y'X*invpd(X'X)*X'Y;


/* Recall that: ginfo=startvalue|stopvalue|number of steps. 
   Recall that: cinfo=startvalue|stopvalue|number of steps.
   Remember also that g=0 gives a linear model. */
   
g=ln(ginfo[1]);                      
gstep=(ln(ginfo[2])-ln(ginfo[1]))/(ginfo[3]-1);
cstep=(cinfo[.,2]-cinfo[.,1])./(cinfo[.,3]-1);


cls;             /* Clear screen */
format /rd 1,0;

if k==1;
    print "Total number of combinations in the grid: ";; 
    sizeM=ginfo[3]*cinfo[1,3];

elseif sym==1;
    print "Total number of combinations in the symmetric grid: ";; 
    sizeM=ginfo[3];
    i=1;
    do until i > k;
        sizeM=sizeM*(cinfo[i,3]-1+i)/i;
        i=i+1;
    endo;

else;
    print "Total number of combinations in the nonsymmetric grid: ";; 
    sizeM=ginfo[3]; 
    i=1;
    do until i > k;
        sizeM=sizeM*cinfo[i,3];
        i=i+1;
    endo;   
endif;
print sizeM;
  
print " ";
print "Percent of the GRID that are calculated: ";;
cx=csrlin;
cy=csrcol;
print 0;;

M=-1*ones(sizeM,2+k);   /*Initalize with negative value, for error check. */
ind=1;

do until g > ln(ginfo[2])+gstep/2;

    if k==1;             /* For speed only. */
        c1=cinfo[1,1];
        do until c1 > cinfo[1,2]+cstep[1]/2; 
            cvec=c1; 
            M[ind,1]=ssrproc(SSR1,SSR2,ARDL,ARN,Y,Ydelay,g,cvec,k,indrest,eps);
            M[ind,2]=exp(g);
            M[ind,3]=cvec;    
            c1=c1+cstep[1];
            ind=ind+1;
        endo; 

    elseif k==2;         /* For speed only. */
        c1=cinfo[1,1];
        do until c1 > cinfo[1,2]+cstep[1]/2;
            if sym==1; c2=c1;
            else;      c2=cinfo[2,1];
            endif;
            do until c2 > cinfo[2,2]+cstep[2]/2; 
                cvec=c1~c2; 
                M[ind,1]=ssrproc(SSR1,SSR2,ARDL,ARN,Y,Ydelay,g,cvec,k,indrest,eps);
                M[ind,2]=exp(g);
                M[ind,3:4]=cvec;
                c2=c2+cstep[2];
                ind=ind+1;
            endo;    
            c1=c1+cstep[1];
        endo; 
        
    

    elseif k==3;         /* For speed only. */
        c1=cinfo[1,1];
        do until c1 > cinfo[1,2]+cstep[1]/2;
            if sym==1; c2=c1;
            else;      c2=cinfo[2,1];
            endif;
            do until c2 > cinfo[2,2]+cstep[2]/2;
                if sym==1; c3=c2;
                else;      c3=cinfo[3,1];
                endif; 
                do until c3 > cinfo[3,2]+cstep[3]/2;
                    cvec=c1~c2~c3; 
                    M[ind,1]=ssrproc(SSR1,SSR2,ARDL,ARN,Y,Ydelay,g,cvec,k,indrest,eps);
                    M[ind,2]=exp(g);
                    M[ind,3:5]=cvec;
                    c3=c3+cstep[3];
                    ind=ind+1;
                endo;
                c2=c2+cstep[2];
            endo;    
            c1=c1+cstep[1];
        endo;         
   

    elseif k==4;         /* For speed only. */
        c1=cinfo[1,1];
        do until c1 > cinfo[1,2]+cstep[1]/2;
            if sym==1; c2=c1;
            else;      c2=cinfo[2,1];
            endif;
            do until c2 > cinfo[2,2]+cstep[2]/2;
                if sym==1; c3=c2;
                else;      c3=cinfo[3,1];
                endif; 
                do until c3 > cinfo[3,2]+cstep[3]/2;
                    if sym==1; c4=c3;
                    else;      c4=cinfo[4,1];
                    endif; 
                    do until c4 > cinfo[4,2]+cstep[4]/2;
                        cvec=c1~c2~c3~c4; 
                        M[ind,1]=ssrproc(SSR1,SSR2,ARDL,ARN,Y,Ydelay,g,cvec,k,indrest,eps);
                        M[ind,2]=exp(g);
                        M[ind,3:6]=cvec;
                        c4=c4+cstep[4];
                        ind=ind+1;
                    endo;
                    c3=c3+cstep[3];
                endo;
                c2=c2+cstep[2];
            endo;    
            c1=c1+cstep[1];
        endo; 


    else;                /* General recursive function call. */
    print "not implemented";
    stop;
        
    endif;


    format /rd 1,0;              /* Updating info. on screen. */
    locate cx,cy;
    print (g-ln(ginfo[1]))/(ln(ginfo[2])-ln(ginfo[1]))*100;;
    format /m1 /rd 12,6;

    g=g+gstep;
endo;

RETP(M);

ENDP; /********************* END OF PROC GRID ************************/

/***************************************************************************/
PROC(1)=ssrproc(SSR1,SSR2,ARDL,ARN,Y,Ydelay,g,cvec,k,indrest,eps);
LOCAL dstd,temp,j,prod,F,Fmean,Nonpart,SSR,x,invert;

dstd=stdc(Ydelay);
trap 1;
/* Calculate the non-linear part of the STAR model */

temp=(Ydelay-cvec[1]);
j=2;
do until j > k;
    temp=temp~(Ydelay-cvec[j]);
    j=j+1;
endo;

prod=prodc(temp');       /* We need the product of the rows */
prod=prod/dstd^k;        /* Making gamma scale invariant */

F=1/(1+EXP(-g*prod));



Fmean=meanc(F);
IF Fmean < eps;                      /* Check if the F-term is to small in */
				     /* average, i.e. use linear model. */
    if scalmiss(indrest)==0;       
        SSR=SSR1;    
    else;
        SSR=SSR2;    
    endif;

ELSEIF Fmean > 1-eps;
    SSR=SSR2;    

ELSE;
    Nonpart=ARN.*F;

    if scalmiss(indrest)==0;           /* If true, there is/are restriction(s). */
        j=1;
        do until j > cols(ARN);
	    if scalmiss(indexcat(indrest,j))==0; /* If true, index in ARN is */
		    			         /* restricted. */
	        Nonpart[.,j]=ARN[.,j].*(F-1);        
	    endif;
	    j=j+1;
       endo;
    endif;

    X=ARDL~Nonpart;                  /* Full model. Include special case  */
    invert=invpd(X'X);
    if scalmiss(invert)==0;
        SSR=Y'Y-Y'X*invert*X'Y;      /* when no linear part, due to the */
    else;
        SSR=SSR2;
    endif;
				     /* representation of the restrictions. */ 
ENDIF;

trap 0;
RETP(SSR);

ENDP; /********************* END OF PROC SSRPROC ************************/





