NEW;                        /* File: STAR.PRG */
cls;

library optmum, Newton, pgraph, starstga, Misc;
dlibrary -a starstga;                            /* Open "C" library */

/*----------------------------------------------------------------------------*/
/*--------------------------- USER SPECIFICATION -----------------------------*/
/*----------------------------------------------------------------------------*/

sel_opt=0;			@ 0: Use optmum in GAUSS, (not implemented 1: Use Newtons algorithm) @
sel_scale=1;		@ 1: Scale to unit,	2: Scale user defined, other#: No scale @
sel_test=1;         @ 0: skip the tests, 1: Compute the tests. @ 

forecast= 1000;		@ save last 1000 observations for forcasts @ 
diff 	= 1;		@ first diff: 1 if used, 0 otherwise @
selday 	= 1;		@ dummies: 1 if present, 0 otherwise @


set_prec=0.000001;
xrate="omx";
	            /*------------ Conditional Mean ------------*/

d=1;						@ Delay parameter for STAR model. @

   plin=4;
pnonlin=0|4;
  restr={};

x=5.6|0.003;					@ Startvalues to inizialize optimization. @

k=rows(x)-1;				@ Order of the STAR model. @

	        /*------------ Conditional Variance ------------*/

alfa0	= 7.7e-6;
alfa	= 0.166;
beta	= 0.836;

STalfa0 =-1.7e-6; 
STalfa0 = {};
STalfa	= -0.115;

slope	= 660;
kvec	=0.012;
kvec	= {};

	       /*--------------- FILE PARAMETERS ---------------*/

	
baseDIR= "c:\\wingauss\\mydir";
savefile = baseDIR $+ "\\" $+ xrate $+ "_d" $+ ftos(d,"%*.*lf",1,0) $+ "LSTAR" $+ ftos(k,"%*.*lf",1,0) $+ ".out";
saveresid= baseDIR $+ "\\" $+ xrate $+ "_d" $+ ftos(d,"%*.*lf",1,0) $+ "LSTAR" $+ ftos(k,"%*.*lf",1,0) $+ "res";


{input,dum,dummies}=getData2(diff,savefile,xrate,selday,forecast);     /* Load data into system */



	       /*-------------- TEST PARAMETERS ---------------*/


qmax=6;        /* Maximum order of serial independence test,
			      used when testing estimated STAR model  */

psup=5;       /* Maximum number of lag for which some apriori
			      is different from 0 used when testing estimated
			      STAR model against remaining non-linearity.
			      Should be greater than the maximum lag defined
			      above.*/

ksup=3;        /* Maximum order of STAR model used as alternative
			      when testing estimated STAR model against
			      remaining non-linearity */
xmax=8;

/*----------------------------------------------------------------------------*/
/*------------------------ END OF USER SPECIFICATION -------------------------*/
/*----------------------------------------------------------------------------*/

T=rows(input);

plin=sortc(plin,1);              /* Precautionary sorting. */
pnonlin=sortc(pnonlin,1);
restr=sortc(restr,1);


pmax=plin[rows(plin)]|pnonlin[rows(pnonlin)];  /* Finding the maximum lag */
p=maxc(pmax|d);


/*---------------- Calculate some parts of the STAR model -------------------*/

	 /*------Adjusting the length of matrix and vectors -------*/

IF scalmiss(Dummies)==0;
    Dummys=Dummies[1+p:rows(input),.];
ELSE;
    Dummys={};
ENDIF;

Y=input[1+p:rows(input)];
Ydelay=input[1+p-d:rows(input)-d];
dstd=stdc(Ydelay);                        /* Variance of delay vector */


		    /*------- Linear AR matrix ----------*/
      /* Used when constructing the linear AR part of the STAR model. */
   
IF plin[1]==0;     /* This condition checks whether a constant is present */
    ARL=ones(rows(input)-p,1);
    i=2;
ELSE;
    ARL=input[1+p-plin[1]:rows(input)-plin[1]];
    i=2;
ENDIF;

do until i > rows(plin);
    ARL=ARL~input[1+p-plin[i]:rows(input)-plin[i]];
    i=i+1;
endo;

orgARL=ARL;

      /*------- Change ARL if restriction(s) is/are present. ---------*/

IF scalmiss(restr)==0;           /* If true, there is/are restriction(s). */
    indrest=indnv(restr,plin);   /* Create a vectorindex for the parameter(s)*/ 
				 /* that are restricted in plin. */
    ARLtemp={};
    j=1;
    do until j > rows(plin); 
	if scalmiss(indexcat(indrest,j))==1; /* If true, there is no restr */
					     /* for index j in plin. */
	    ARLtemp=ARLtemp~ARL[.,j];
	endif;
	j=j+1;
    endo;
    ARL=ARLtemp; 
ENDIF;


ARDL=Dummys~ARL;    /* Matrix of Dummy matrix and AR matrix */


		 /*-------- Non-linear AR matrix ---------*/
     /* Used when constructing the non-linear AR part of the STAR model */

IF pnonlin[1]==0;     /* This condition checks whether a constant is present */
    ARN=ones(rows(input)-p,1);
    i=2;
ELSE;
    ARN=input[1+p-pnonlin[1]:rows(input)-pnonlin[1]];
    i=2;
ENDIF;

do until i > rows(pnonlin);
    ARN=ARN~input[1+p-pnonlin[i]:rows(input)-pnonlin[i]];
    i=i+1;
endo;

orgARN=ARN;

/*---------------- Define INISIZE --------------*/ 

insize=zeros(9,1);
IF scalmiss(dummys)==0;				@ size Dummies @
    insize[1]=cols(Dummys);
ELSE;
    insize[1]=0;
ENDIF;

insize[2]=cols(ARL);				@ size Linear  @
insize[3]=cols(ARN);				@ size Nonlinear @
insize[4]=k;

insize[5]=rows(alfa);				@ sizeAlfa variance GARCH(alfa,beta) @
insize[6]=rows(beta);				@ sizeBeta variance @

if ismiss(kvec);
	insize[7]=0;					@ location set to zero apriori @
else;
	insize[7]=rows(kvec);			@ location(s) in estimation @
endif;

if ismiss(stalfa0);
	insize[8]=0;					@ parameters set to zero apriori @
else;
	insize[8]=rows(stalfa0);		@ parameter(s) in estimation @
endif;

if ismiss(stalfa);
	insize[9]=0;					@ parameters set to zero apriori @
else;
	insize[9]=rows(stalfa);			@ parameter(s) in estimation @
endif;


/*---------------- Initialvalues for the estimation  ------------------*/

sizeo=insize[1]+insize[2]+insize[3]+1+insize[4];
sizea=insize[5];
sizeb=insize[6];
sizee=1+sizea+sizeb;

/* The previous estimate of the Gamma and the Constant(s) are used. */
temp=rows(x);
menpar=x[temp-k:temp];     
menpar=inival(menpar,ARL,ARN,Dummys,Y,Ydelay,dstd,k,restr,plin)|menpar;

if ismiss(kvec);
	varpar=alfa0|alfa|beta|stalfa0|stalfa|slope;          
else;
	varpar=alfa0|alfa|beta|stalfa0|stalfa|slope|kvec;          
endif;

start=menpar|varpar;

/*	Choose how to scale the parametervalues */
if sel_scale==1;						@ Scale to unity. @
	use_scale=1/start;			
elseif sel_scale==2;					@ User defined scale. @
	if rows(start) ne rows(use_scale);
		print " ";
		print "The # of modelparameters and # of scaleparameters differs!";
		print "Take care of this problem in the initialization.";
		stop; 
	endif; 
else;									@ No scale. @
	use_scale=ones(rows(start),1);
endif;									@ Do the scaleing. @

start=start.*use_scale;	

/* Transform for positive variance parameters */
start[sizeo+1:sizeo+2]            =start[sizeo+1:sizeo+2].^0.5;
start[sizeo+1+sizea+1:sizeo+sizee]=start[sizeo+1+sizea+1:sizeo+sizee].^0.5;

namevec=parnames(k,plin,pnonlin,restr,dum,alfa,beta,stalfa0,stalfa,kvec);


/*----------------------- Numerical optimizations --------------------------*/
Nonpart=zeros(rows(ARN),cols(ARN));				/* For speed only, used by ssrstar */


IF sel_opt==0;
    _opparnm=namevec;							/* Create vector for parameternames */
	_optol=set_prec;							/* Select the tolerance */  
    {x,f,g,retcode}=optmum(&lik_StarStga,start);
    hessian=-hessp(&lik_StarStga,x);			/* Numerical hessian */
	numbit=ftos(_opitdta[1],"%*.*lf",5,0);
    tottime=ftos(_opitdta[2],"%*.*lf",10,5);
ELSEIF sel_opt==1;
	print "Not implemented, use sel_opt=0 instead";
	stop;
ENDIF;


/*------------------ Untransform the estimated parameters.------------------*/
								  
@ Transform => initial and estimated parametervalues. @
    x[sizeo+1:sizeo+2]                =x[sizeo+1:sizeo+2].^2;
    x[sizeo+1+sizea+1:sizeo+sizee]    =x[sizeo+1+sizea+1:sizeo+sizee].^2;
start[sizeo+1:sizeo+2]            =start[sizeo+1:sizeo+2].^2;
start[sizeo+1+sizea+1:sizeo+sizee]=start[sizeo+1+sizea+1:sizeo+sizee].^2;

x		=x    ./use_scale;
start	=start./use_scale;

/*---------------------------------------------------------------------------*/



restricted_x=x;
{stdres}=residual(x);

likvalue=f;
                          
{dl,hessian}=finhess(x);
covar=inv(-hessian);




   /*---- Calculate some statistics for the estimated STAR-model --------*/

 /* Following Tersvirta, Mellin 1986, model selection criteria and model
    selection tests in regressions models, Scand J Statist 13:159-171
    T-p        number of observation's
    rows(x)    number of free parameters in the model */

AIC=ln(f/(T-p))+2*rows(x)/(T-p);            /* Akaike, 1974 */
SBIC=ln(f/(T-p))+ln(T-p)*rows(x)/(T-p);     /* Schwartz 1978, Rissanen 1978 */
HQ=ln(f/(T-p))+2*rows(x)*ln(ln(T-p))/(T-p); /* Hannan & Quinn 1979 */

SSY=(Y-meanc(Y))'*(Y-meanc(Y));
R2=1-f/SSY;
adjR2=1-(f/(T-p))/(SSY/(T-1));

hhh=zeros(rows(x),rows(x));
hd=diag(covar);
ett=ones(rows(x),1);
hdi=ett./hd;
hdinv=diagrv(hhh,hdi);
hdinv=sqrt(hdinv);
correl=hdinv*covar*hdinv;             /* The correlation matrix */

inpvar=dstd^2;
xvar=diag(covar);
ttest=x./sqrt(xvar);
tprob=2*cdftc(abs(ttest),T-p-rows(x));  /* Double sided test */
estimate=start~x~sqrt(xvar)~ttest~tprob;


   /*------------ Create t-test for new restriction(s) ---------------*/
rsum={};
rvar={};
rindex={};
templin=plin;



IF ismiss(restr)==0;        /* Remove all present restriction(s) from */
    i=1;                      /* the linear polynomia. */
    do until i > rows(restr);
	templin=delif(templin,templin[.,1] .eq restr[i]);
	i=i+1;
    endo;
ENDIF;

IF scalmiss(dum)==0;          /* If daily data. */
   j=rows(dum);
ELSE;
   j=0;
ENDIF;

L=seqa(1,1,rows(templin))~templin~estimate[j+1:j+rows(templin),.];
N=seqa(1,1,rows(pnonlin))~pnonlin;
N=N~estimate[j+rows(templin)+1:j+rows(templin)+rows(pnonlin),.];

i=0;                                 /* Check if equal then it's a candidate */
do until i > p;                      /* for new restriction(s). */
    a=selif(L,L[.,2] .eq i);              /* Linear part. */
    b=selif(N,N[.,2] .eq i);              /* Non-linear part. */
    if scalmiss(a)==0 and scalmiss(b)==0;
       rindex=rindex|a[2];
       rsum=rsum|(a[4]+b[4]);             /* Sum of the two parameters. */
       temp=a[5]*a[5]+b[5]*b[5];          /* Variance of the sum. */
       rvar=rvar|(temp+2*a[5]*b[5]*correl[j+a[1],j+rows(templin)+b[1]]);
    endif;
    i=i+1;
endo;


rttest=rsum./sqrt(rvar);                 /* t-value. */
rtprob=2*cdftc(abs(rttest),T-p-rows(x)); /* obs frihetsgrader kollas */
rest=rindex~rsum~sqrt(rvar)~rttest~rtprob;

   /*-------------- Adjustment because of restriction(s) -------------*/

IF scalmiss(restr)==0;      /* If there are restriction(s). */
    restlin=indnv(restr,plin);
    restnon=indnv(restr,pnonlin);
  
    if scalmiss(dummys)==0; /* If true, daily data. */
	sizeD=cols(Dummys);
	newest=estimate[1:sizeD,.];
    else;
	sizeD=0;
	newest={};        
    endif;

    i=sizeD+1;
    j=1;
    do until j > rows(plin);
	if scalmiss(indexcat(restlin,j))==0; /* If true, rest. for index j */            
	    temp=-estimate[sizeD+rows(plin)-rows(restr)+indexcat(pnonlin,plin[j]),.];
	    temp[.,3]=-temp[.,3];
	    temp[.,5]=-temp[.,5];
	    newest=newest|temp;
	else;                                /* Else, no rest. for index j. */
	    newest=newest|estimate[i,.];
	    i=i+1;
	endif;        
	j=j+1;
    endo;

    newest=newest|estimate[i:rows(estimate),.];
    estimate=newest;
    x=estimate[.,2];
ENDIF;

   /*----------------- Printing the result (to file) -------------------*/

output file=^savefile on;



/*--------------- Print the estimated model and its inference ---------------*/

scalar_miss={};
namevec=parnames(k,plin,pnonlin,scalar_miss,dum,alfa,beta,stalfa0,stalfa,kvec);	@ Unrestricted namevector @

print " ";
print "The estimated parameters of the STAR-STGARCH model.";
print " ";
print "            Initial       Estimated       Std dev        t-test      p-value";
mask=ones(1,6);
mask[1]=0;
let fmt[6,3] =
           "-*.*s " 6 6  
           "*.*le " 14 4 
           "*.*le " 14 4
           "*.*le " 14 4
           "*.*lf " 12 6
           "*.*lf " 12 6;
out=namevec~estimate;
k=printfm(out,mask,fmt);
print " "; 
print_moments(stdres);


/*	INIZIALIZE THE TESTS
	-We need the estimated model with the restrictions used in the estimation. 
	-"plin" must be without restrictions, since the test against additional
	 nonlinearity also use eventual "holes".
	-Insize is the same as above. 
*/

x=restricted_x;
plotF(x);

ARDL=Dummys~ARL;
{u,ARNF,F}=obtainF(x);
OMX2=F;
save OMX2;

if sel_test==1;
	format /rd 10,6;
/*
	print " ";
	print "----------------------- Conditional mean part ------------------------";
	print " ";

	meanSERI(x,insize,ARDL,ARN,ARNF,Ydelay,Y,qmax);

  	meanNONL(x,insize,ARDL,ARN,ARNF,Ydelay,Y,plin,5,1);
 	meanNONL(x,insize,ARDL,ARN,ARNF,Ydelay,Y,plin,5,2);
 	meanNONL(x,insize,ARDL,ARN,ARNF,Ydelay,Y,plin,5,3);

	meanCONS(x,insize,ARDL,ARN,ARNF,Ydelay,Y,1);
	meanCONS(x,insize,ARDL,ARN,ARNF,Ydelay,Y,2);
	meanCONS(x,insize,ARDL,ARN,ARNF,Ydelay,Y,3);
*/
	print " ";
	print "--------------------- Conditional variance part ----------------------";
	print " ";

	 serARCH(x,insize,ARDL,ARN,ARNF,Ydelay,Y,10);
	serLiMak(x,insize,ARDL,ARN,ARNF,Ydelay,Y,4);
	  serADD(x,insize,ARDL,ARN,ARNF,Ydelay,Y,10);
    serBOLLu(x,insize,ARDL,ARN,ARNF,Ydelay,Y,4);
	serBOLLg(x,insize,ARDL,ARN,ARNF,Ydelay,Y,4);

	  nonGJR(x,insize,ARDL,ARN,ARNF,Ydelay,Y);
	nonDelay(x,insize,ARDL,ARN,ARNF,Ydelay,Y,3);
	nonSignB(x,insize,ARDL,ARN,ARNF,Ydelay,Y);

	 parcons(x,insize,ARDL,ARN,ARNF,Ydelay,Y,1);
	 parcons(x,insize,ARDL,ARN,ARNF,Ydelay,Y,2);
	 parcons(x,insize,ARDL,ARN,ARNF,Ydelay,Y,3);

endif;


print "-----------------------------------------------------------------------";
print "---------------------------- END OF MODEL -----------------------------";
print "-----------------------------------------------------------------------";
output off;

dlibrary -d; 
END; /********************* END OF STAR.PGM **************************/


/**************************************************************************
 * Procedure INIVAL                                                       *
 *                                                                        *
 * This procedure calculate initial estimates of the linear AR polynom    *
 * and the non-linear AR polynom.                                         *
 *                                                                        *
 * Arguments of the procedure are:                                        *
 *                                                                        *
 * in          A row vector that contains  Gamma and the Constant(s).     *
 * ARL         The linear AR part of the STAR model.                      *
 * ARN         The non-linear AR part of the STAR model.                  *
 * Y, Ydelay   Observation vectors.                                       *
 * dstd        Standard deviation of the delay vector.                    *
 * k           Order of the STAR model                                    *
 * restr       The restriction vector.                                    *
 * plin        Apriori for linear AR polynomia.                           *
 * ARDL        ARDL=Dummys~ARL;                                           *
 *                                                                        *
 * The procedure returns the estimated linear and non-linear polynom of   *
 * the STAR model given Gamma and the Constant(s).                        *
 *************************************************************************/

PROC(1)=inival(in,ARL,ARN,Dummys,Y,Ydelay,dstd,k,restr,plin);
LOCAL X,i,j,F,temp,prod,beta,betaN,Fmean,templin,templ,tempnon,tempn,indrest;
LOCAL g,c,betaL,nonpart,ARR,invxx;

trap 1;

/* Identifying  */

g=in[1];
c=in[2:k+1];

/* Calculate the non-linear part of the STAR model */

temp=(Ydelay-c[1]);
j=2;
do until j > k;
    temp=temp~(Ydelay-c[j]);
    j=j+1;
endo;

prod=prodc(temp');                   /* We need the product of the rows */
prod=prod/dstd^k;                    /* Making gamma scale invariant */

F=1/(1+EXP(-g*prod));

Nonpart=ARN.*F;

IF scalmiss(restr)==0;           /* If true, there is/are restriction(s). */
    indrest=indnv(restr,pnonlin);/* Create a vectorindex for the parameter(s)*/
				                 /* that are restricted in plin. */
    j=1;
    do until j > cols(ARN);
	if scalmiss(indexcat(indrest,j))==0; /* If true, index in ARN is restricted.*/
	    Nonpart[.,j]=ARN[.,j].*(F-1);        
	endif;
	j=j+1;
    endo;
ENDIF;

ARDL=Dummys~ARL;

X=ARDL~Nonpart;                  /* Full model. Include special case when no linear part, */
                                 /* due to the representation of the restrictions. */  
invxx=invpd(X'X);
if scalmiss(invxx)==0;   /* If true, the inversion is possible. */ 
    beta=invxx*X'Y;      
else;                    /* If not, the matrix is singuar, can't inverse */
    Fmean=meanc(F);
    if Fmean < 0.5;      /* F-term is close to 0, i.e. use linear model. */
        IF scalmiss(restr)==0;
            ARR=-ARN[.,indnv(restr,pnonlin)];
            X=ARDL~ARR;
            beta=invpd(X'X)*X'Y;    
            temp=beta[cols(ARDL)+1:cols(ARDL)+rows(restr)];
            betaN=zeros(cols(ARN),1);
            j=1;
            i=1;
            do until j > cols(ARN);
	        if scalmiss(indexcat(restr,j))==0;
	            betaN[j]=temp[i];
	            i=i+1;
	        endif;
	        j=j+1;
            endo;
            beta=beta[1:cols(ARDL)]|betaN;
        ELSE;
            X=ARDL;          
            beta=invpd(X'X)*X'Y;
            betaN=zeros(cols(ARN),1);
            beta=beta|betaN;
        ENDIF;
    else;                 /* F-term is close to 1, i.e. use linear model. */
        X=ARDL;
        beta=invpd(X'X)*X'Y;    
        betaN=zeros(cols(ARN),1);
        beta=beta|betaN;
    endif;
endif;			    

trap 0;
	   
RETP(beta);                          /* Return estimated startvalues. */

ENDP; /******************* END OF PROC INIVAL *************************/

/**************************************************************************
 * Procedure PARNAMES                                                     *
 *                                                                        *
 * This procedure create a namevector for the GENERAL NONLINEAR           *
 * ESTIMATION routine.                                                    *
 *                                                                        *
 * Arguments of the procedure are:                                        *
 *                                                                        *
 * k           Order of the STAR model.                                   *
 * plin        Apriori for linear AR polynomia.                           *
 * pnonlin     Apriori for nonlinear AR polynomia.                        *
 * restr       The restriction vector.                                    *
 * dum         String with names of dummy variables.                      *
 *                                                                        *
 * The procedure returns a vector with string elements                    *
 *************************************************************************/

PROC(1)=parnames(k,plin,pnonlin,restr,dum,alfa,beta,stalfa0,stalfa,kvec);

LOCAL namevec,j,lintemp,nontemp;

nontemp=(0$+"ar("$+ftocv(pnonlin,1,0)$+")N"); /* Nonlinear parameternames. */
 
IF scalmiss(restr)==0;
    j=1;
    do until j > rows(restr);
       nontemp[indexcat(pnonlin,restr[j])]=(0$+"ar("$+ftocv(restr[j],1,0)$+")R");
       plin=delif(plin,plin[.,1] .eq restr[j]);
       j=j+1;
    endo;
ENDIF;

IF scalmiss(plin)==1;
    lintemp={};
ELSE;
    lintemp=(0$+"ar("$+ftocv(plin,1,0)$+")L"); /* Linear parameternames. */
ENDIF;

namevec=dum|lintemp|nontemp|"Gamma"|(0$+"Const"$+ftocv(seqa(1,1,k),1,0));

namevec=namevec|"alfa0"|(0$+"alfa1"$+ftocv(seqa(1,1,rows(alfa)),1,0));
namevec=namevec|(0$+"beta"$+ftocv(seqa(1,1,rows(beta)),1,0));

j=1;
do until j > rows(alfa);
	if ismiss(stalfa0) ne 1;
    	namevec=namevec|0$+"alfa0"$+ftocv(j,1,0);
	endif;
	if ismiss(stalfa) ne 1;
    	namevec=namevec|0$+"alfa2"$+ftocv(j,1,0);
	endif;
    j=j+1;
endo;

namevec=namevec|"slope";

if ismiss(kvec) ne 1;
	namevec=namevec|(0$+"k"$+ftocv(seqa(1,1,rows(kvec)),1,0));
endif;

RETP(namevec);
ENDP; /******************* END OF PROC PARNAMES *************************/


/**************************************************************************
 * Procedure resSTAR                                                      *
 *                                                                        *
 * Given a STAR model with constant variance this procedure construct the *
 * likelihoodfunction. Used in Powell's algorithm.                        *
 *                                                                        *
 * in        A row vector that contains all parameter-values required to  *
 *           identify the STAR model. Gamma, Constant(s), the linear AR   *
 *           polynom (incl. the constant) and the non-linear AR polynom.  *
 * ARL       The linear AR part of the STAR model.                        *
 * orgARL    Used to speed up calculations.                               *
 * ARN       The non-linear AR part of the STAR model.                    *
 * Dummys    Dummyvariables used if data is daily.                        *
 * Y, Ydelay Observation vectors.                                         *
 * dstd      Standard deviation of the delay vector.                      *
 * k         Order of the STAR model                                      *
 * restr     The restriction vector.                                      *
 * plin      Apriori for linear AR polynomia.                             *
 *                                                                        *
 * The procedure returns the value of the likelihoodfunction for a given  *
 * parameter vector.                                                      *
 *************************************************************************/

PROC(2)=resSTAR(in);
LOCAL temp,j,prod,F,indrest,STAR,residual,betaD,betaL,betaN,betaDL,sizeKm;
LOCAL g,c,sizeD,sizeL,sizeN,var,likeli,Nonpart;

 sizeD=insize[1];
 sizeL=insize[2];
 sizeN=insize[3];
sizeKm=insize[4];


g=in[sizeD+sizeL+sizeN+1];
c=in[sizeD+sizeL+sizeN+2:sizeD+sizeL+sizeN+1+sizeKm];

betaN=in[sizeD+sizeL+1:sizeD+sizeL+sizeN];                         

/* Calculate the non-linear part of the STAR model */

temp=(Ydelay-c[1]);
j=2;
do until j > sizeKm;
    temp=temp~(Ydelay-c[j]);
    j=j+1;
endo;

prod=prodc(temp');       /* We need the product of the rows */
prod=prod/dstd^sizeKm;        /* Making gamma scale invariant */

F=1/(1+EXP(-g*prod));

Nonpart=zeros(rows(ARN),cols(ARN));  

Nonpart=ARN.*F;

IF scalmiss(restr)==0;           /* If true, there is/are restriction(s). */
    indrest=indnv(restr,pnonlin);/* Create a vectorindex for the parameter(s)*/
				                 /* that are restricted in pnonlin. */
    j=1;
    do until j > cols(ARN);
	if scalmiss(indexcat(indrest,j))==0; /* If true, index in ARN is restricted. */
	    Nonpart[.,j]=ARN[.,j].*(F-1);        
	endif;
	j=j+1;
    endo;
ENDIF;

IF scalmiss(ARL)==1 and scalmiss(Dummys)==1;    
    STAR=Nonpart*betaN;    
ELSE;
    betaDL=in[1:sizeD+sizeL];
    STAR=ARDL*betaDL+Nonpart*betaN;            
ENDIF;

residual=Y-STAR;

retp(residual,Nonpart);

ENDP; /********************* END OF PROC resSTAR **************************/


PROC(1)=g_StarStga(in);
LOCAL sizeD,sizeL,sizeN,sizeKm,sizeo;
LOCAL sizeA,sizeB,sizeST,sizeK,sizee,sizev;
LOCAL ARDL,ARNF,u,indll,initial,dstd;
LOCAL i,meanGRAD,meanHESS,elements,premult,postadd;

/* Use of transformation Y=X.^2  */

 sizeD=insize[1];
 sizeL=insize[2];
 sizeN=insize[3];
sizeKm=insize[4];
 sizeA=insize[5];
 sizeB=insize[6];
 sizeK=insize[7];
 
 sizee=1+sizeA+sizeB;
 sizeo=sizeD+sizeL+sizeN+1+sizeKm;
sizeST=insize[8]+insize[9];
 sizev=sizee+sizeST+1+sizeK;

ARDL=Dummys~ARL;
meanGRAD=zeros(1,sizeo+sizev); 

indll							   =in;   
indll[sizeo+1:sizeo+2]             =in[sizeo+1:sizeo+2].^2;
indll[sizeo+1+sizea+1:sizeo+sizee] =in[sizeo+1+sizea+1:sizeo+sizee].^2;
indll							   =indll./use_scale;

{u,ARNF}=resSTAR(indll);  /* Calculate residuals for the cond. mean */
dstd=stdc(Ydelay);

initial	=rows(ARDL)|0|sizeD+sizeL|sizeN|sizeKm|dstd|sizeA|sizeB|sizeK|insize[8]|insize[9]|0;

dllcall  _gST(initial,indll,ARDL,ARN,ARNF,Y,Ydelay,meanGRAD);


/*	Adjust for the squared terms	=>	 ensure positive values 
	Adjust for the scaling			=>	 faster convergence */

premult=diagrv(zeros(rows(indll),rows(indll)),1./use_scale);

elements=sizeo+1|sizeo+2|sizeo+1+sizea+seqa(1,1,sizeb);

i=1;		
do until i > rows(elements);
    /* Construct dY/dX:	*/
    premult[elements[i],elements[i]]=premult[elements[i],elements[i]]*2*in[elements[i]];
    i=i+1;
endo;

meanGRAD=meanGRAD*premult;

retp(-meanGRAD); 
endp;

/*---------------------------------------------------------------------------*/

/*--------------------------------------------------------------------*/

PROC(2)=h_StarStga(in);
LOCAL sizeD,sizeL,sizeN,sizeKm,sizeo;
LOCAL sizeA,sizeB,sizeST,sizeK,sizee,sizev;
LOCAL ARDL,ARNF,u,indll,initial,dstd;
LOCAL i,meanGRAD,meanHESS,elements,premult,postadd;


/* Use of transformation Y=X.^2  */

 sizeD=insize[1];
 sizeL=insize[2];
 sizeN=insize[3];
sizeKm=insize[4];
 sizeA=insize[5];
 sizeB=insize[6];
 sizeK=insize[7];
 
 sizee=1+sizeA+sizeB;
 sizeo=sizeD+sizeL+sizeN+1+sizeKm;
sizeST=insize[8]+insize[9];
 sizev=sizee+sizeST+1+sizeK;


ARDL=Dummys~ARL;


indll							   =in;   
indll[sizeo+1:sizeo+2]             =in[sizeo+1:sizeo+2].^2;
indll[sizeo+1+sizea+1:sizeo+sizee] =in[sizeo+1+sizea+1:sizeo+sizee].^2;
indll							   =indll./use_scale;

meanGRAD=zeros(1,sizeo+sizev); 
meanHESS=zeros(sizeo+sizev,sizeo+sizev);

{u,ARNF}=resSTAR(indll);  /* Calculate residuals for the cond. mean */
dstd=stdc(Ydelay);

initial	=rows(ARDL)|0|sizeD+sizeL|sizeN|sizeKm|dstd|sizeA|sizeB|sizeK|insize[8]|insize[9]|0;

dllcall  _hST(initial,indll,ARDL,ARN,ARNF,Y,Ydelay,meanGRAD,meanHESS);


/*	Adjust for the squared terms	=>	 ensure positive values 
	Adjust for the scaling			=>	 faster convergence */

premult=diagrv(zeros(rows(in),rows(in)),1./use_scale);
postadd= zeros(rows(in),rows(in));

elements=sizeo+1|sizeo+2|sizeo+1+sizea+seqa(1,1,sizeb);

i=1;		
do until i > rows(elements);
    /* Construct dY/dX:	*/
    premult[elements[i],elements[i]]=premult[elements[i],elements[i]]*2*in[elements[i]];
    /* Construct derivative of vec((df/dY)*(dY/dX)) with respect to X */
    postadd[elements[i],elements[i]]=2*meanGRAD[elements[i]]./use_scale[elements[i]];
    i=i+1;
endo;

meanGRAD=meanGRAD*premult;
meanHESS=premult*meanHESS*premult+postadd;

retp(-meanGRAD,-meanHESS); 
endp;

/*---------------------------------------------------------------------------*/

/*--------------------------------------------------------------------*/

PROC(2)=finhess(in);
LOCAL sizeD,sizeL,sizeN,sizeKm,sizeo;
LOCAL sizeA,sizeB,sizeST,sizeK,sizee,sizev;
LOCAL ARDL,ARNF,u,indll,initial,dstd;
LOCAL meanhess,meangrad;

 sizeD=insize[1];
 sizeL=insize[2];
 sizeN=insize[3];
sizeKm=insize[4];
 sizeA=insize[5];
 sizeB=insize[6];
 sizeK=insize[7];
 
 sizee=1+sizeA+sizeB;
 sizeo=sizeD+sizeL+sizeN+1+sizeKm;
sizeST=insize[8]+insize[9];
 sizev=sizee+sizeST+1+sizeK;


ARDL=Dummys~ARL;

meanGRAD=zeros(1,sizeo+sizev); 
meanHESS=zeros(sizeo+sizev,sizeo+sizev);

{u,ARNF}=resSTAR(in);  /* Calculate residuals for the cond. mean */
dstd=stdc(Ydelay);

initial	=rows(ARDL)|0|sizeD+sizeL|sizeN|sizeKm|dstd|sizeA|sizeB|sizeK|insize[8]|insize[9]|0;

dllcall  _hST(initial,in,ARDL,ARN,ARNF,Y,Ydelay,meanGRAD,meanHESS);


retp(meanGRAD,meanHESS); 
endp;

/*---------------------------------------------------------------------------*/

PROC(1)=gausslik_StarStga(in);
LOCAL sizeD,sizeL,sizeN,sizeA,sizeB,sizeK,sizeST,sizeKm;
LOCAL sizee,sizeo,sizev,dstd;
LOCAL ARDL,ARNF,u,likhod,initial;
LOCAL nuobs,stdu,prod,i,j,H,temp,g,maxab,u2,maxof1andk;
LOCAL presamp_u,presamp_g,presamp_ARL,presamp_uARL,presamp_prod,presamp_H;
LOCAL alfa0,alfa,beta,STa0,STa,locat,slope;

/* Use of transformation Y=X.^2  */

 sizeD=insize[1];
 sizeL=insize[2];
 sizeN=insize[3];
sizeKm=insize[4];
 sizeA=insize[5];
 sizeB=insize[6];
 sizeK=insize[7];
 
sizee=1+sizeA+sizeB;
sizeo=sizeD+sizeL+sizeN+1+sizeKm;
sizeST=insize[8]+insize[9];
sizev=sizee+sizeST+1+sizeK;

maxof1andk	=maxc(1|sizek);
maxab		=maxc(sizea|sizeb);

ARDL=Dummys~ARL;
likhod=0;
 
in[sizeo+1:sizeo+2]             =in[sizeo+1:sizeo+2].^2;
in[sizeo+1+sizea+1:sizeo+sizee] =in[sizeo+1+sizea+1:sizeo+sizee].^2;
in							    =in./use_scale;


alfa0		= in[sizeo+1];
alfa		= in[sizeo+2:sizeo+sizea];
beta		= in[sizeo+1+sizea+1:sizeo+1+sizea+sizeb];
if (insize[8] ne 0);
	STa0	= in[sizeo+sizee+1:sizeo+sizee+sizea];
	temp	= sizea;
else;
	STa0	= zeros(1,sizea);
	temp	= 0;
endif;
if (insize[9] ne 0);
	STa		= in[sizeo+sizee+temp+1:sizeo+sizee+temp+sizea];
else;
	STa		= zeros(1,sizea);
endif;
slope		= in[sizeo+sizee+sizeST+1];
if (sizek ne 0);
	locat	= in[sizeo+sizee+sizeST+2:sizeo+sizee+sizeST+1+sizek];
else;
	locat	= 0;
endif;


{u,ARNF}=resSTAR(in);  /* Calculate residuals for the cond. mean */
dstd=stdc(Ydelay);

nuobs=rows(ARDL);

u2			= u.^2;
presamp_u	= meanc(u);
presamp_g	= meanc(u2);
/*
presamp_ARL	= meanc(ARDL)';
presamp_uARL= meanc(u.*ARDL)';
*/
stdu		= 1;

prod		= ones(nuobs,1).*((1/stdu).^maxof1andk);
presamp_prod= 1.*((1/stdu).^maxof1andk);

j=1;
do until j > maxof1andk;
	presamp_prod=presamp_prod*(presamp_u-locat[j]);
	prod		=prod.*(u-locat[j]);
	j=j+1;
endo;

H=zeros(nuobs,1);

temp		= 0.5*slope*presamp_prod;
presamp_H	= (0.5*exp(temp))/cosh(temp);

temp		= 0.5*(slope.*prod);
H			= (0.5*(exp(temp))./cosh(temp));

g			= zeros(nuobs,1);

i=1;
do until i > nuobs;
	g[i]=alfa0;
	j=1;
	do until j > maxab;

		if (j <= sizea);
			if ((i-j) > 0);
				g[i]=g[i]+alfa[j]*u2[i-j]	+(STa0[j]+STa[j]*u2[i-j])	*H[i-j];
			else;
				g[i]=g[i]+alfa[j]*presamp_g	+(STa0[j]+STa[j]*presamp_g)	*presamp_H;
			endif;
		endif;

		if (j <= sizeb);
			if ((i-j) > 0);
				g[i]=g[i]+beta[j]*g[i-j];
			else;
				g[i]=g[i]+beta[j]*presamp_g;
			endif;
		endif;
		
		j=j+1;
	endo;
	i=i+1;
endo;


likhod=sumc(-0.5*ln(g)-0.5*(u2./g));

/*retp(-likhod);*/ 				/* We want to minimize instead of maximize */

retp(-likhod);
endp;

/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/

PROC(1)=lik_StarStga(in);
LOCAL sizeD,sizeL,sizeN,sizeA,sizeB,sizeK,sizeST,sizeKm;
LOCAL sizee,sizeo,sizev,dstd;
LOCAL ARDL,ARNF,u,likhod,indll,initial;

/* Use of transformation Y=X.^2  */

 sizeD=insize[1];
 sizeL=insize[2];
 sizeN=insize[3];
sizeKm=insize[4];
 sizeA=insize[5];
 sizeB=insize[6];
 sizeK=insize[7];
 
sizee=1+sizeA+sizeB;
sizeo=sizeD+sizeL+sizeN+1+sizeKm;
sizeST=insize[8]+insize[9];
sizev=sizee+sizeST+1+sizeK;

ARDL=Dummys~ARL;
likhod=0;

indll							   =in;   
indll[sizeo+1:sizeo+2]             =in[sizeo+1:sizeo+2].^2;
indll[sizeo+1+sizea+1:sizeo+sizee] =in[sizeo+1+sizea+1:sizeo+sizee].^2;
indll							   =indll./use_scale;

{u,ARNF}=resSTAR(indll);  /* Calculate residuals for the cond. mean */
dstd=stdc(Ydelay);

initial	=rows(ARDL)|0|sizeD+sizeL|sizeN|sizeKm|dstd|sizeA|sizeB|sizeK|insize[8]|insize[9]|0;

dllcall  _likST(initial,indll,ARDL,ARN,ARNF,Y,Ydelay,likhod);

retp(-likhod); 				/* We want to minimize instead of maximize */
endp;

/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/

PROC(1)=fin_lik_StarStga(in);
LOCAL sizeD,sizeL,sizeN,sizeA,sizeB,sizeK,sizeST,sizeKm;
LOCAL sizee,sizeo,sizev,dstd;
LOCAL ARDL,ARNF,u,likhod,indll,initial;

 sizeD=insize[1];
 sizeL=insize[2];
 sizeN=insize[3];
sizeKm=insize[4];
 sizeA=insize[5];
 sizeB=insize[6];
 sizeK=insize[7];
 
sizee=1+sizeA+sizeB;
sizeo=sizeD+sizeL+sizeN+1+sizeKm;
sizeST=insize[8]+insize[9];
sizev=sizee+sizeST+1+sizeK;

ARDL=Dummys~ARL;
likhod=0;


{u,ARNF}=resSTAR(in);  /* Calculate residuals for the cond. mean */
dstd=stdc(Ydelay);

initial	=rows(ARDL)|0|sizeD+sizeL|sizeN|sizeKm|dstd|sizeA|sizeB|sizeK|insize[8]|insize[9]|0;

dllcall  _likST(initial,in,ARDL,ARN,ARNF,Y,Ydelay,likhod);

retp(-likhod); 				/* We want to minimize instead of maximize */
endp;

/*---------------------------------------------------------------------------*/


/*--------------------------------------------------------------------*/
/*------------------------- Plotting F -------------------------------*/
/*--------------------------------------------------------------------*/

PROC(0)=plotF(in);
local u,a,ARNF,F,info,mycolor;
{u,ARNF,F}=obtainF(in);                   /* Calculate residuals for the cond. mean */

graphset;
mycolor  = 0;          
_pcsel   = ones(7,1)*mycolor;      @ Default colour vector @
_pmcolor = (ones(8,1)*mycolor)|15; @ Axes, labels, etc. @
_pcolor  = mycolor;                @ Main curves in XY, XYZ, and LOG graphs @
_pzclr   = mycolor;                @ SURFACE graphs @
_pdate   = "";                     @ Remove datestring @
_plctrl=-1;
_pstype=1;
_pnum=2;
ytics(0,1,0.10,0.05);
xy(ydelay,f);

endp;

/*--------------------------------------------------------------------*/

PROC(3)=obtainF(in);
LOCAL temp,j,prod,F,indrest,STAR,residual,betaD,betaL,betaN,betaDL;
LOCAL g,c,sizeD,sizeL,sizeN,var,likeli,Nonpart;

sizeD=insize[1];
sizeL=insize[2];
sizeN=insize[3];

g=in[sizeD+sizeL+sizeN+1];
c=in[sizeD+sizeL+sizeN+2:sizeD+sizeL+sizeN+1+k];

betaN=in[sizeD+sizeL+1:sizeD+sizeL+sizeN];                         

/* Calculate the non-linear part of the STAR model */

temp=(Ydelay-c[1]);
j=2;
do until j > k;
    temp=temp~(Ydelay-c[j]);
    j=j+1;
endo;

prod=prodc(temp');       /* We need the product of the rows */
prod=prod/dstd^k;        /* Making gamma scale invariant */

F=1/(1+EXP(-g*prod));

Nonpart=zeros(rows(ARN),cols(ARN));  

Nonpart=ARN.*F;

IF scalmiss(ARL)==1 and scalmiss(Dummys)==1;    
    STAR=Nonpart*betaN;    
ELSE;
    betaDL=in[1:sizeD+sizeL];
    STAR=ARDL*betaDL+Nonpart*betaN;            
ENDIF;

residual=Y-STAR;

retp(residual,Nonpart,F);

ENDP;
/*--------------------------------------------------------------------*/

PROC(1)=residual(in);
LOCAL sizeD,sizeL,sizeN,sizeA,sizeB,sizeK,sizeST,sizeKm;
LOCAL sizee,sizeo,sizev,dstd;
LOCAL ARDL,ARNF,u,initial,indll,stdu;

 sizeD=insize[1];
 sizeL=insize[2];
 sizeN=insize[3];
sizeKm=insize[4];
 sizeA=insize[5];
 sizeB=insize[6];
 sizeK=insize[7];
 
sizee=1+sizeA+sizeB;
sizeo=sizeD+sizeL+sizeN+1+sizeKm;
sizeST=insize[8]+insize[9];
sizev=sizee+sizeST+1+sizeK;

ARDL=Dummys~ARL;
stdu=zeros(rows(ARDL),1);


{u,ARNF}=resSTAR(in);  /* Calculate residuals for the cond. mean */
dstd=stdc(Ydelay);

initial	=rows(ARDL)|0|sizeD+sizeL|sizeN|sizeKm|dstd|sizeA|sizeB|sizeK|insize[8]|insize[9]|0;

dllcall  _modres(initial,in,ARDL,ARN,ARNF,Y,Ydelay,stdu);

retp(stdu); 				/* Return the standardized residuals */
endp;

/*---------------------------------------------------------------------------*/














