NEW;                        /* File: STAR.PRG */
cls;
library optmum,newton,STARtst;			/* Open GAUSS librarys */ 
dlibrary -a star;						/* Open "C" library */

/*----------------------------------------------------------------------------*/
/*--------------------------- USER SPECIFICATION -----------------------------*/
/*----------------------------------------------------------------------------*/

sel_opt=1;			@ 0: Use optmum in GAUSS, 1: Use Newtons algorithm @
sel_scale=0;		@ 1: Scale to unit, 2:Scale user defined, other#: No scale @
sel_loop=0;         @ 0: Compute the specified model, 1: Automatic elimination. @ 
sel_test=1;         @ 0: Skip the tests, 1: Compute the tests. @ 

forecast= 0;		@ save last 0 observations for forcasts @ 
diff 	= 0;		@ first diff: 1 if used, 0 otherwise @
selday	= 0;		@ dummies: 1 if present, 0 otherwise @

Series= "sp500";       @ Chose asset to analyse. @

	       /*------------ GENERAL PARAMETERS ------------*/

d=2;						@ Delay parameter for STAR model. @

   plin=1|2|4|5|6|8;		@ Lags present in "linear" AR part (0 denotes a constant) @
pnonlin=1|2|4|5;			@ Lags present in "nonlinear" AR part (0 denotes a constant) @
  restr=1|5;				@ Impose restrictions between these paramters @

x=2|0.022;					@ slope|c1|c2|....@

k=rows(x)-1;				@ Order of the STAR model. @


	       /*--------------- FILE PARAMETERS ---------------*/

startype="d" $+ ftos(d,"%*.*lf",1,0) $+ "LSTAR" $+ ftos(k,"%*.*lf",1,0); 
baseDIR= "C:\\wingauss\\mydir";
savefile= baseDIR $+ "\\" $+ series $+ "_" $+ startype $+ ".txt";
saveresid= baseDIR $+ "\\" $+ series $+ "_" $+ startype;


{input,dum,dummies}=getData2(diff,savefile,Series,selday,forecast);


	    /*-------------- OPTIMIZATION PARAMETERS ---------------*/

prec=0.000001;         /* Set precision for parmeter estimate */

	       /*-------------- TEST PARAMETERS ---------------*/


qmax=8;                /* Maximum order of serial independence test,
			              used when testing estimated STAR model  */

psup=8;               /* Maximum number of lag for which some apriori
			              is different from 0 used when testing estimated
			              STAR model against remaining non-linearity.
			              Should be greater than the maximum lag defined
			              above.*/

ksup=3;                /* Maximum order of STAR model used as alternative
			              when testing estimated STAR model against
			              remaining non-linearity */

/*----------------------------------------------------------------------------*/
/*------------------------ END OF USER SPECIFICATION -------------------------*/
/*----------------------------------------------------------------------------*/


T=rows(input);

plin=sortc(plin,1);              /* Precautionary sorting. */
pnonlin=sortc(pnonlin,1);
restr=sortc(restr,1);



DO WHILE rows(plin)>=1 and rows(pnonlin)>=1;   /* This is a LOOP which is used
						  for automatic elimination of
						  the model. */


pmax=plin[rows(plin)]|pnonlin[rows(pnonlin)];  /* Finding the maximum lag */
p=maxc(pmax|d);


/*---------------- Calculate some parts of the STAR model -------------------*/

	 /*------Adjusting the length of matrix and vectors -------*/

IF scalmiss(Dummies)==0;
    Dummys=Dummies[1+p:rows(input),.];
ELSE;
    Dummys={};
ENDIF;

Y=input[1+p:rows(input)];
Ydelay=input[1+p-d:rows(input)-d];
dstd=stdc(Ydelay);                        /* Variance of delay vector */


		    /*------- Linear AR matrix ----------*/
      /* Used when constructing the linear AR part of the STAR model. */
   
IF plin[1]==0;     /* This condition checks whether a constant is present */
    ARL=ones(rows(input)-p,1);
    i=2;
ELSE;
    ARL=input[1+p-plin[1]:rows(input)-plin[1]];
    i=2;
ENDIF;

do until i > rows(plin);
    ARL=ARL~input[1+p-plin[i]:rows(input)-plin[i]];
    i=i+1;
endo;

orgARL=ARL;

      /*------- Change ARL if restriction(s) is/are present. ---------*/

IF scalmiss(restr)==0;           /* If true, there is/are restriction(s). */
    indrest=indnv(restr,plin);   /* Create a vectorindex for the parameter(s)*/ 
				 /* that are restricted in plin. */
    ARLtemp={};
    j=1;
    do until j > rows(plin); 
	if scalmiss(indexcat(indrest,j))==1; /* If true, there is no restr */
					     /* for index j in plin. */
	    ARLtemp=ARLtemp~ARL[.,j];
	endif;
	j=j+1;
    endo;
    ARL=ARLtemp; 
ENDIF;


ARDL=Dummys~ARL;    /* Matrix of Dummy matrix and AR matrix */


		 /*-------- Non-linear AR matrix ---------*/
     /* Used when constructing the non-linear AR part of the STAR model */

IF pnonlin[1]==0;     /* This condition checks whether a constant is present */
    ARN=ones(rows(input)-p,1);
    i=2;
ELSE;
    ARN=input[1+p-pnonlin[1]:rows(input)-pnonlin[1]];
    i=2;
ENDIF;

do until i > rows(pnonlin);
    ARN=ARN~input[1+p-pnonlin[i]:rows(input)-pnonlin[i]];
    i=i+1;
endo;

orgARN=ARN;


/*---------------------- startvalues  ------------------------*/

                       /* The previous estimate of the Gamma and the */
     temp=rows(x);          /* Constant(s) are used. */
     start=x[temp-k:temp];

/*---------------- Define INISIZE --------------*/ 


insize=zeros(5,1);
IF scalmiss(dummies)==0;           /* sizeD */
    insize[1]=cols(Dummys);
ELSE;
    insize[1]=0;
ENDIF;

insize[2]=cols(ARL);              /* sizeL */
insize[3]=cols(ARN);              /* sizeN */
insize[4]=0;                      /* sizeA */
insize[5]=0;                      /* sizeB */



/*----------------------- Numerical optimizations --------------------------*/
Nonpart=zeros(rows(ARN),cols(ARN));				/* For speed only, used by ssrstar */

start=inival(start,ARL,ARN,Dummys,Y,Ydelay,dstd,k,restr,plin)|start;	/* Appending linear and non-linear polynom to start vector. */

/*	Choose how to scale the parametervalues */
if sel_scale==1;						@ Scale to unity. @
	scale=1/start;			
elseif sel_scale==2;					@ User defined scale. @
	if rows(start) ne rows(scale);
		print " ";
		print "The # of modelparameters and # of scaleparameters differs!";
		print "Take care of this problem in the initialization.";
		stop; 
	endif; 
else;									@ No scale. @
	scale=ones(rows(start),1);
endif;									@ Do the scaleing. @

start=start.*scale;	

namevec=parnames(k,plin,pnonlin,restr,dum);		/* Create vector for parameternames */


IF sel_opt==0;     
    _opparnm=namevec;							/* Create vector for parameternames */
	_optol=prec;							/* Select the tolerance */  
    {x,f,g,retcode}=optmum(&ssrstar,start);
/*	hessian=_opfhess; */
    hessian=hessp(&ssrstar,x);					/* Numerical hessian */
	numbit=ftos(_opitdta[1],"%*.*lf",5,0);
    tottime=ftos(_opitdta[2],"%*.*lf",10,5);

ELSEIF sel_opt==1;
    tottime=hsec;
    _parnm=namevec;								/* Create vector for parameternames */
    _tol=prec;								/* Select the tolerance */
    {x,f,g,hessian,numbit}=Newton(&ssrstar,&hessproc,start);
    g=g';										/* To correspond with optmum conventions */
    tottime=(hsec-tottime)/6000;

ENDIF;

x=x./scale;	
start=start./scale;	

@ Untransformed values @
scale=ones(rows(x),1);
{dl,hessian}=hessproc(x);

{modres,nonpart}=resSTAR(x);

resvar=f/(T-p-rows(x)); /* Variance of the residuals. */
covar=resvar*inv(hessian);
ARL=orgARL;
ARN=orgARN;



   /*---- Calculate some statistics for the estimated STAR-model --------*/

 /* Following Tersvirta, Mellin 1986, model selection criteria and model
    selection tests in regressions models, Scand J Statist 13:159-171
    T-p        number of observation's
    rows(x)    number of free parameters in the model */

AIC=ln(f/(T-p))+2*rows(x)/(T-p);            /* Akaike, 1974 */
SBIC=ln(f/(T-p))+ln(T-p)*rows(x)/(T-p);     /* Schwartz 1978, Rissanen 1978 */
HQ=ln(f/(T-p))+2*rows(x)*ln(ln(T-p))/(T-p); /* Hannan & Quinn 1979 */


SSY=(Y-meanc(Y))'*(Y-meanc(Y));
R2=1-f/SSY;
adjR2=1-(f/(T-p))/(SSY/(T-1));

hhh=zeros(rows(x),rows(x));
hd=diag(covar);
ett=ones(rows(x),1);
hdi=ett./hd;
hdinv=diagrv(hhh,hdi);
hdinv=sqrt(hdinv);
correl=hdinv*covar*hdinv;             /* The correlation matrix */

inpvar=dstd^2;
xvar=diag(covar);
ttest=x./sqrt(xvar);
tprob=2*cdftc(abs(ttest),T-p-rows(x));  /* Double sided test */
estimate=start~x~sqrt(xvar)~ttest~tprob;

   /*------------ Create t-test for new restriction(s) ---------------*/
rsum={};
rvar={};
rindex={};
templin=plin;

IF scalmiss(restr)==0;        /* Remove all present restriction(s) from */
    i=1;                      /* the linear polynomia. */
    do until i > rows(restr);
	templin=delif(templin,templin[.,1] .eq restr[i]);
	i=i+1;
    endo;
ENDIF;

IF scalmiss(dum)==0;          /* If daily data. */
   j=rows(dum);
ELSE;
   j=0;
ENDIF;

L=seqa(1,1,rows(templin))~templin~estimate[j+1:j+rows(templin),.];
N=seqa(1,1,rows(pnonlin))~pnonlin;
N=N~estimate[j+rows(templin)+1:j+rows(templin)+rows(pnonlin),.];

i=0;                                 /* Check if equal then it's a candidate */
do until i > p;                      /* for new restriction(s). */
    a=selif(L,L[.,2] .eq i);              /* Linear part. */
    b=selif(N,N[.,2] .eq i);              /* Non-linear part. */
    if scalmiss(a)==0 and scalmiss(b)==0;
       rindex=rindex|a[2];
       rsum=rsum|(a[4]+b[4]);             /* Sum of the two parameters. */
       temp=a[5]*a[5]+b[5]*b[5];          /* Variance of the sum. */
       rvar=rvar|(temp+2*a[5]*b[5]*correl[j+a[1],j+rows(templin)+b[1]]);
    endif;
    i=i+1;
endo;


rttest=rsum./sqrt(rvar);                 /* t-value. */
rtprob=2*cdftc(abs(rttest),T-p-rows(x)); /* obs frihetsgrader kollas */
rest=rindex~rsum~sqrt(rvar)~rttest~rtprob;

   /*-------------- Adjustment because of restriction(s) -------------*/

IF scalmiss(restr)==0;      /* If there are restriction(s). */
    restlin=indnv(restr,plin);
    restnon=indnv(restr,pnonlin);
   
    if scalmiss(dummies)==0; /* If true, daily data. */
	sizeD=cols(Dummys);
	newest=estimate[1:sizeD,.];
    else;
	sizeD=0;
	newest={};        
    endif;

    i=sizeD+1;
    j=1;
    do until j > rows(plin);
	if scalmiss(indexcat(restlin,j))==0; /* If true, rest. for index j */            
	    temp=-estimate[sizeD+rows(plin)-rows(restr)+indexcat(pnonlin,plin[j]),.];
	    temp[.,3]=-temp[.,3];
	    temp[.,5]=-temp[.,5];
	    newest=newest|temp;
	else;                                /* Else, no rest. for index j. */
	    newest=newest|estimate[i,.];
	    i=i+1;
	endif;        
	j=j+1;
    endo;

    newest=newest|estimate[i:rows(estimate),.];
    estimate=newest;
    x=estimate[.,2];
ENDIF;

   /*----------------- Printing the result (to file) -------------------*/

output file=^savefile on;

prtmodel(d,k,plin,pnonlin,restr,dum,estimate,rest);
print "----------- Some Information about the estimated STAR model -----------";
print " "; 
format /ld 5,0;   
print "Number of iterations                         " numbit;				     
format /m1 /rd 10,6;
print "Minutes to convergence                       " tottime;				     
print "AIC                                          " AIC;
print "SBIC                                         " SBIC;
print "HQ                                           " HQ;
print "R2                                           " R2;
print "Adjusted R2                                  " adjR2;
print "Variance of the delay vector                 " inpvar;
print "Standard deviation of the delay vector       " dstd;
print "Variance of the residual vector              " resvar;
print "Standard deviation of the residual vector    " sqrt(resvar);
print " ";


IF sel_test==1;
	print "-----------------------------------------------------------------------";
	trap 1;										/* Turn off safety belt. */
	print " ";

	insize[2]=cols(ARL);              /* sizeL */
	insize[3]=cols(ARN);              /* sizeN */

	{resid,ARNF,Z,ARNL,Holes,ARL,ARNF}=preTest(x,insize,Dummys~ARL,ARN,Y,Ydelay,dstd,k,p,psup,input,plin,pnonlin,restr);

	SERIND(resid,Z,8);

	REMNOLIN(Holes,ARNL,trimr(resid,psup-p,0),trimr(z,psup-p,0),ksup);

	PARACONS(DUMMYS~ARL~ARNF,resid,Z,ksup,"All",1);
	PARACONS(ARL			,resid,Z,ksup,"Lin",0);
	PARACONS(ARNF			,resid,Z,ksup,"Non",0);

	print " ";
	trap 0;										/* Turn on safety belt. */

	print "-----------------------------------------------------------------------";
ENDIF;


save ^saveresid=modres;


print "-----------------------------------------------------------------------";
print "---------------------------- END OF MODEL -----------------------------";
print "-----------------------------------------------------------------------";
output off;


   /*------------- The automatic elimination algorithm ----------------*/

/* Here we eliminate the linear or the non-linear estimate with the largest
   probability value, the alternative is that the element is 0. The model
   is re-estimated without that estimate, i.e. apriori=0. The algorithm is
   terminated when there is only 1 parameter left in the linear or in the
   non-linear parameter vector. */


IF sel_loop==1;

    if scalmiss(dummies)==0; /* If true, daily data. */
	sizeD=cols(Dummys);
    else;
	sizeD=0;        
    endif;
    sizeL=rows(plin);
    sizeN=rows(pnonlin);

    templin=plin~estimate[1+sizeD:sizeD+sizeL,5];
    maxlin=maxc(templin);
    tempnon=pnonlin~estimate[1+sizeD+sizeL:sizeD+sizeL+sizeN,5];
    maxnon=maxc(tempnon);
    max=maxc(maxlin[2]|maxnon[2]);

    if scalmiss(rest)==0;        /* Check if there are restriction(s). */
	maxrest=maxc(rest[.,5]);
    else;
	maxrest=0;
    endif;

    IF max > maxrest;       /* Eliminate a lag and/or a restriction. */

	/* If we should, eliminate a lag from "plin". */
	restlin=selif(templin,templin[.,2] .ge max);
	templin=delif(templin,templin[.,2] .ge max);
	if scalmiss(templin)==1;
	    plin={};
	else;
	    plin=templin[.,1];
	endif;

	/* If we should, eliminate a lag from "pnonlin". */
	restnon=selif(tempnon,tempnon[.,2] .ge max);
	tempnon=delif(tempnon,tempnon[.,2] .ge max);
	if scalmiss(tempnon)==1;
	    pnonlin={};
	else;
	    pnonlin=tempnon[.,1];
	endif;

	/* If we should, eliminate a restriction. */
	if scalmiss(restlin)==0;
	   if restlin==restnon;
			if rows(restr)==1;
				restr={};
			else;
				restr=delif(restr,restr .eq restlin[.,1]);
			endif;
	   endif;
	endif;

    ELSE;                             /* introduce a restriction */

	resttemp=selif(rest,rest[.,5] .ge maxrest);
	restr=restr|resttemp[1];
	restr=sortc(restr,1);

    ENDIF;

ELSE;
    plin={};      /* When the loop are different from 0 */
    pnonlin={};   /* we terminate the program. */
ENDIF;

ENDO;             /* End of do-loop administrating the automatic elimination */


dlibrary -d;								/* Close "C" library */    

END; /********************* END OF STAR.PGM **************************/


/**************************************************************************
 * Procedure SSRSTAR                                                      *
 *                                                                        *
 * This procedure are used by OPTMUM and calculate the Square Sum of      *
 * Residuals given that we have all parameter-values. Because of OPTMUM   *
 * this procedure can only take the parameter-values as arguments. As a   *
 * result of this we must use some "global" variables.                    *
 *                                                                        *
 * Arguments of the procedure are:                                        *
 *                                                                        *
 * in        A row vector that contains all parameter-values required to  *
 *           identify the STAR model. Gamma, Constant(s), the linear AR   *
 *           polynom (incl. the constant) and the non-linear AR polynom.  *
 *                                                                        *
 * The "global" variables are:                                            *
 *                                                                        *
 * ARL         The linear AR part of the STAR model.                      *
 * orgARL      Used to speed up calculations.                             *
 * ARN         The non-linear AR part of the STAR model.                  *
 * Dummys      Dummyvariables used if data is daily.                      *
 * Y, Ydelay   Observation vectors.                                       *
 * dstd        Standard deviation of the delay vector.                    *
 * k           Order of the STAR model                                    *
 * restr       The restriction vector.                                    *
 * plin        Apriori for linear AR polynomia.                           *
 * ARDL        ARDL=Dummys~ARL;                                           *
 *                                                                        *
 * The procedure returns the scalar SSR.                                  *
 *************************************************************************/

PROC(1)=ssrstar(in);
LOCAL temp,j,prod,F,indrest,STAR,residual,betaD,betaL,betaN,betaDL;
LOCAL g,c,sizeD,sizeL,sizeN;

in =in./scale;      

sizeD=insize[1];
sizeL=insize[2];
sizeN=insize[3];

g=in[sizeD+sizeL+sizeN+1];
c=in[sizeD+sizeL+sizeN+2:sizeD+sizeL+sizeN+1+k];

betaN=in[sizeD+sizeL+1:sizeD+sizeL+sizeN];                         

/* Calculate the non-linear part of the STAR model */

temp=(Ydelay-c[1]);
j=2;
do until j > k;
    temp=temp~(Ydelay-c[j]);
    j=j+1;
endo;

prod=prodc(temp');       /* We need the product of the rows */
prod=prod/dstd^k;        /* Making gamma scale invariant */

F=1/(1+EXP(-g*prod));

/* Nonpart=zeros(rows(ARN),cols(ARN));  i huvudrutin for hastighet */ 

Nonpart=ARN.*F;

IF scalmiss(restr)==0;           /* If true, there is/are restriction(s). */
    indrest=indnv(restr,pnonlin);/* Create a vectorindex for the parameter(s)*/
				                 /* that are restricted in pnonlin. */
    j=1;
    do until j > cols(ARN);
	if scalmiss(indexcat(indrest,j))==0; /* If true, index in ARN is restricted. */
	    Nonpart[.,j]=ARN[.,j].*(F-1);        
	endif;
	j=j+1;
    endo;
ENDIF;

IF scalmiss(ARL)==1 and scalmiss(Dummys)==1;    
    STAR=Nonpart*betaN;    
ELSE;
    betaDL=in[1:sizeD+sizeL];
    STAR=ARDL*betaDL+Nonpart*betaN;            
ENDIF;

residual=Y-STAR;

RETP(residual'*residual);

ENDP; /****************** END OF PROC SSRSTAR ***********************/

/**************************************************************************
 * Procedure INIVAL                                                       *
 *                                                                        *
 * This procedure calculate initial estimates of the linear AR polynom    *
 * and the non-linear AR polynom.                                         *
 *                                                                        *
 * Arguments of the procedure are:                                        *
 *                                                                        *
 * in          A row vector that contains  Gamma and the Constant(s).     *
 * ARL         The linear AR part of the STAR model.                      *
 * ARN         The non-linear AR part of the STAR model.                  *
 * Y, Ydelay   Observation vectors.                                       *
 * dstd        Standard deviation of the delay vector.                    *
 * k           Order of the STAR model                                    *
 * restr       The restriction vector.                                    *
 * plin        Apriori for linear AR polynomia.                           *
 * ARDL        ARDL=Dummys~ARL;                                           *
 *                                                                        *
 * The procedure returns the estimated linear and non-linear polynom of   *
 * the STAR model given Gamma and the Constant(s).                        *
 *************************************************************************/

PROC(1)=inival(in,ARL,ARN,Dummys,Y,Ydelay,dstd,k,restr,plin);
LOCAL X,i,j,F,temp,prod,beta,betaN,Fmean,templin,templ,tempnon,tempn,indrest;
LOCAL g,c,betaL,nonpart,ARR,invxx;

trap 1;

/* Identifying  */

g=in[1];
c=in[2:k+1];

/* Calculate the non-linear part of the STAR model */

temp=(Ydelay-c[1]);
j=2;
do until j > k;
    temp=temp~(Ydelay-c[j]);
    j=j+1;
endo;

prod=prodc(temp');                   /* We need the product of the rows */
prod=prod/dstd^k;                    /* Making gamma scale invariant */

F=1/(1+EXP(-g*prod));

Nonpart=ARN.*F;

IF scalmiss(restr)==0;           /* If true, there is/are restriction(s). */
    indrest=indnv(restr,pnonlin);/* Create a vectorindex for the parameter(s)*/
				                 /* that are restricted in plin. */
    j=1;
    do until j > cols(ARN);
	if scalmiss(indexcat(indrest,j))==0; /* If true, index in ARN is restricted.*/
	    Nonpart[.,j]=ARN[.,j].*(F-1);        
	endif;
	j=j+1;
    endo;
ENDIF;

ARDL=Dummys~ARL;

X=ARDL~Nonpart;                  /* Full model. Include special case when no linear part, */
                                 /* due to the representation of the restrictions. */  
invxx=invpd(X'X);
if scalmiss(invxx)==0;   /* If true, the inversion is possible. */ 
    beta=invxx*X'Y;      
else;                    /* If not, the matrix is singuar, can't inverse */
    Fmean=meanc(F);
    if Fmean < 0.5;      /* F-term is close to 0, i.e. use linear model. */
        IF scalmiss(restr)==0;
            ARR=-ARN[.,indnv(restr,pnonlin)];
            X=ARDL~ARR;
            beta=invpd(X'X)*X'Y;    
            temp=beta[cols(ARDL)+1:cols(ARDL)+rows(restr)];
            betaN=zeros(cols(ARN),1);
            j=1;
            i=1;
            do until j > cols(ARN);
	        if scalmiss(indexcat(restr,j))==0;
	            betaN[j]=temp[i];
	            i=i+1;
	        endif;
	        j=j+1;
            endo;
            beta=beta[1:cols(ARDL)]|betaN;
        ELSE;
            X=ARDL;          
            beta=invpd(X'X)*X'Y;
            betaN=zeros(cols(ARN),1);
            beta=beta|betaN;
        ENDIF;
    else;                 /* F-term is close to 1, i.e. use linear model. */
        X=ARDL;
        beta=invpd(X'X)*X'Y;    
        betaN=zeros(cols(ARN),1);
        beta=beta|betaN;
    endif;
endif;			    

trap 0;
	   
RETP(beta);                          /* Return estimated startvalues. */

ENDP; /******************* END OF PROC INIVAL *************************/

/**************************************************************************
 * Procedure PARNAMES                                                     *
 *                                                                        *
 * This procedure create a namevector for the GENERAL NONLINEAR           *
 * ESTIMATION routine.                                                    *
 *                                                                        *
 * Arguments of the procedure are:                                        *
 *                                                                        *
 * k           Order of the STAR model.                                   *
 * plin        Apriori for linear AR polynomia.                           *
 * pnonlin     Apriori for nonlinear AR polynomia.                        *
 * restr       The restriction vector.                                    *
 * dum         String with names of dummy variables.                      *
 *                                                                        *
 * The procedure returns a vector with string elements                    *
 *************************************************************************/

PROC(1)=parnames(k,plin,pnonlin,restr,dum);
LOCAL namevec,j,lintemp,nontemp;

nontemp=(0$+"ar("$+ftocv(pnonlin,1,0)$+")N"); /* Nonlinear parameternames. */
 
IF scalmiss(restr)==0;
    j=1;
    do until j > rows(restr);
       nontemp[indexcat(pnonlin,restr[j])]=(0$+"ar("$+ftocv(restr[j],1,0)$+")R");
       plin=delif(plin,plin[.,1] .eq restr[j]);
       j=j+1;
    endo;
ENDIF;

IF scalmiss(plin)==1;
    lintemp={};
ELSE;
    lintemp=(0$+"ar("$+ftocv(plin,1,0)$+")L"); /* Linear parameternames. */
ENDIF;

namevec=dum|lintemp|nontemp|"Gamma"|(0$+"Const"$+ftocv(seqa(1,1,k),1,0));

RETP(namevec);
ENDP; /******************* END OF PROC PARNAMES *************************/

/**************************************************************************
 * Procedure PRTMODEL                                                     *
 *                                                                        *
 * This procedure print the estimated model.                              *
 *                                                                        *
 * Arguments of the procedure are:                                        *
 *                                                                        *
 * d           Delay parameter for STAR model.                            *
 * k           Order of the STAR model.                                   *
 * plin        Apriori for linear AR polynomia.                           *
 * pnonlin     Apriori for nonlinear AR polynomia.                        *
 * restr       The restriction vector.                                    *
 * dum         String with names of dummy variables.                      *
 * estimate    Estimated parameters with variance and t-test.             *
 * rest        Test result for potential restriction(s).                  *
 *                                                                        *
 * No returns from the procedure.                                         *
 *************************************************************************/

PROC(0)=prtmodel(d,k,plin,pnonlin,restr,dum,estimate,rest);
LOCAL temp,temp1,j,Q,text,i,cvec,gvec,dsize;

format /rd 2,0;
print "Value of the delay parameter (d): "  d;
print "Order of STAR model (k):          "  k;

/* To achieve "nice" shape of plin in the output file */
temp=rows(plin);
if temp <= 12;
    print "Linear AR parameters:             " $ftocv(plin',1,0);
else;
    print "Linear AR parameters:             " $ftocv(plin[1:12]',1,0);
    temp1=(temp-temp%12)/12;
    j=1;
    do until j > temp1;
	if temp <= (j+1)*12;
    print "                                  " $ftocv(plin[1+(j*12):temp]',1,0);
	else;
print "                                  " $ftocv(plin[1+(j*12):(j+1)*12]',1,0);
	endif;
	j=j+1;
    endo;
endif;

/* To achieve "nice" shape of pnonlin in the output file */
temp=rows(pnonlin);
if temp <= 12;
    print "Nonlinear AR parameters:          " $ftocv(pnonlin',1,0);
else;
    print "Nonlinear AR parameters:          " $ftocv(pnonlin[1:12]',1,0);
    temp1=(temp-temp%12)/12;
    j=1;
    do until j > temp1;
	if temp <= (j+1)*12;
    print "                                  "
	  $ftocv(pnonlin[1+(j*12):temp]',1,0);
	else;
    print "                                  "
	  $ftocv(pnonlin[1+(j*12):(j+1)*12]',1,0);
	endif;
	j=j+1;
    endo;
endif;

/* To achieve "nice" shape of restr in the output file */
IF scalmiss(restr)==0;
    temp=rows(restr);
    if temp <= 12;
	print "Restricted parameter(s):          " $ftocv(restr',1,0);
    else;
	print "Restricted parameter(s):          " $ftocv(restr[1:12]',1,0);
	temp1=(temp-temp%12)/12;
	j=1;
	do until j > temp1;
	    if temp <= (j+1)*12;
	print "                                  "
	      $ftocv(restr[1+(j*12):temp]',1,0);
	    else;
	print "                                  "
	      $ftocv(restr[1+(j*12):(j+1)*12]',1,0);
	    endif;
	    j=j+1;
	endo;
    endif;
ELSE;
    print "No restricted parameter(s).";
ENDIF;

/* Define two strings, for frequent output text. */
Q="---------------------------------------------------------------------------";
text="Estimated    Std dev       t-test      p-value";

IF scalmiss(Dummys)==0;
	dsize=rows(dum);
ELSE;
	dsize=0;
ENDIF;

format /m1 /rd 12,6; 
print " ";
print Q;    /*---------------------------------------------------*/
print "         Initial      " text;
print "Gamma" estimate[dsize+rows(plin)+rows(pnonlin)+1,.];

j=dsize+rows(plin)+rows(pnonlin)+2;
i=j-1;
cvec=0$+"C"$+ftocv(seqa(1,1,k),1,0);
do until j > i+rows(cvec);
    format /m1 /rd 4,0;
    print $cvec[j-i];;
    format /m1 /rd 12,6;
    print estimate[j,.];
    j=j+1;
endo;
print Q;    /*---------------------------------------------------*/


IF scalmiss(Dummys)==0; /* Daily data. */
    print "Dummyvariables for daily data:";
    print " ";
	j=1;
    do until j > rows(dum);
	format /m1 /rd 4,0;
	print $dum[j];;
	format /m1 /rd 12,6;
	print estimate[j,.];
	j=j+1;
    endo;
    print Q;    /*---------------------------------------------------*/
ENDIF;

print "Linear AR parameters:    ";
print " ";
print " Lag     Initial      " text;
j=dsize+1;
i=j-1;
do until j > i+rows(plin);
    format /m1 /rd 4,0;
    print plin[j-i];;
    format /m1 /rd 12,6;
    print estimate[j,.];
    j=j+1;
endo;
print Q;    /*---------------------------------------------------*/

print "Non-linear AR parameters: ";
print " ";
print " Lag     Initial      " text;
j=dsize+rows(plin)+1;
i=j-1;
do until j > i+rows(pnonlin);
    format /m1 /rd 4,0;
    print pnonlin[j-i];;
    format /m1 /rd 12,6;
    print estimate[j,.];
    j=j+1;
endo;
print Q;    /*---------------------------------------------------*/

IF scalmiss(Rest)==0;
    print "Test for new restriction(s) in the lag: ";
    print " ";
    print " Lag                  " text;
    j=1;
    do until j > rows(rest);
	format /m1 /rd 4,0;
	print rest[j,1];;
		print "             ";;
	format /m1 /rd 12,6;
	print rest[j,2:cols(rest)]; 
	j=j+1;
    endo;
    print Q;    /*---------------------------------------------------*/
    print " ";
ELSE;
    print "No further restriction possible: ";
    print " ";
ENDIF;
ENDP; /******************* END OF PROC PRTMODEL *************************/


/**************************************************************************
 * Procedure GRADPROC                                                     *
 *                                                                        *
 * This procedure are used by OPTMUM and calculate the Square Sum of      *
 * Residuals given that we have all parameter-values. Because of OPTMUM   *
 * this procedure can only take the parameter-values as arguments. As a   *
 * result of this we must use some "global" variables.                    *
 *                                                                        *
 * Arguments of the procedure are:                                        *
 *                                                                        *
 * in        A row vector that contains all parameter-values required to  *
 *           identify the STAR model. Dummys, the linear AR polynom       *
 *           (incl. the constant), the non-linear AR polynom (incl. the   *
 *           constant), Gamma, and Constant(s).                           * 
 *                                                                        *
 * The "global" variables are:                                            *
 *                                                                        *
 * ARL         The linear AR part of the STAR model.                      *
 * orgARL      Used to speed up calculations.                             *
 * ARN         The non-linear AR part of the STAR model.                  *
 * Dummys      Dummyvariables used if data is daily.                      *
 * Y, Ydelay   Observation vectors.                                       *
 * dstd        Standard deviation of the delay vector.                    *
 * k           Order of the STAR model                                    *
 * restr       The restriction vector.                                    *
 * plin        Apriori for linear AR polynomia.                           *
 * ARDL        ARDL=Dummys~ARL;                                           *
 *                                                                        *
 * The procedure returns the scalar SSR.                                  *
 *************************************************************************/

PROC(1)=gradproc(in);
LOCAL u,initial,meanGRAD,dl,ARNF;

{u,ARNF}=resSTAR(in);

initial=k|dstd|rows(ARN)|cols(ARL)|cols(ARN);
meanGRAD=zeros(1,cols(ARL)+cols(ARN)+k+1);                 /* mean gradient */

dllcall _gstco(initial,in,ARL,ARN,ARNF,Y,Ydelay,u,meanGRAD);

RETP(meanGRAD);

ENDP; /******************* END OF PROC GRADPROC *************************/

/*-------------- Algorithms regarding gradient and Hessian ------------------*/

/**************************************************************************
 * Procedure HESSPROC                                                     *
 *                                                                        *
 * This procedure are used by OPTMUM and calculate the Square Sum of      *
 * Residuals given that we have all parameter-values. Because of OPTMUM   *
 * this procedure can only take the parameter-values as arguments. As a   *
 * result of this we must use some "global" variables.                    *
 *                                                                        *
 * Arguments of the procedure are:                                        *
 *                                                                        *
 * in        A row vector that contains all parameter-values required to  *
 *           identify the STAR model. Dummys, the linear AR polynom       *
 *           (incl. the constant), the non-linear AR polynom (incl. the   *
 *           constant), Gamma, and Constant(s).                           * 
 *                                                                        *
 * The "global" variables are:                                            *
 *                                                                        *
 * ARL         The linear AR part of the STAR model.                      *
 * orgARL      Used to speed up calculations.                             *
 * ARN         The non-linear AR part of the STAR model.                  *
 * Dummys      Dummyvariables used if data is daily.                      *
 * Y, Ydelay   Observation vectors.                                       *
 * dstd        Standard deviation of the delay vector.                    *
 * k           Order of the STAR model                                    *
 * restr       The restriction vector.                                    *
 * plin        Apriori for linear AR polynomia.                           *
 * ARDL        ARDL=Dummys~ARL;                                           *
 *                                                                        *
 * The procedure returns the scalar SSR.                                  *
 *************************************************************************/

PROC(2)=hessproc(in);
LOCAL initial,u,ARNF,meanGRAD,meanHESS,ARDL,premult;

in =in./scale; 
{u,ARNF}=resSTAR(in);

ARDL=Dummys~ARL;
initial=k|dstd|rows(ARN)|cols(ARDL)|cols(ARN);
meanGRAD=zeros(1,cols(ARDL)+cols(ARN)+k+1); 
meanHESS=zeros(cols(ARDL)+cols(ARN)+k+1,cols(ARDL)+cols(ARN)+k+1);

dllcall _hstco(initial,in,ARDL,ARN,ARNF,Y,Ydelay,u,meanGRAD,meanHESS);


/*	Adjust for the squared terms	=>	 insure positive values 
	Adjust for the scaling			=>	 faster convergence */

premult=diagrv(zeros(rows(in),rows(in)),1./scale);

meanGRAD=meanGRAD*premult;
meanHESS=premult*meanHESS*premult;


retp(meanGRAD,meanHESS);


ENDP; /******************* END OF PROC HESSPROC *************************/


/*--------------------------------------------------------------------*/
PROC(2)=hstga(in);
LOCAL initial,u,g,z,ARNF,meanGRAD,meanHESS;
LOCAL sizeD,sizeL,sizeN,sizeA,sizeB,const,alfa,beta;

sizeD=insize[1];                   /* Identify size of model parameters */
sizeL=insize[2];
sizeN=insize[3];
sizeA=insize[4];
sizeB=insize[5];
const=in[sizeD+sizeL+sizeN+1+k+1];  /* Relevant model parameters */
 alfa=in[sizeD+sizeL+sizeN+1+k+2:sizeD+sizeL+sizeN+1+k+1+sizeA];
 beta=in[sizeD+sizeL+sizeN+1+k+1+sizeA+1:sizeD+sizeL+sizeN+1+k+1+sizeA+sizeB];

{u,ARNF}=resSTAR(in);
{g,z}=getgz(u,const,alfa,beta);
initial=k|dstd|rows(ARN)|sizeL|sizeN|sizeA|sizeB;
meanGRAD=zeros(1,sizeL+sizeN+k+2+sizeA+sizeB); 
meanHESS=zeros(sizeL+sizeN+k+2+sizeA+sizeB,sizeL+sizeN+k+2+sizeA+sizeB);

dllcall  _hstga(initial,in,ARL,ARN,ARNF,Y,Ydelay,z,g,u,meanGRAD,meanHESS);

retp(meanGRAD,meanHESS);
/*--------------------------------------------------------------------*/


ENDP; 

/**************************************************************************
 * Procedure LIKSTAR                                                      *
 *                                                                        *
 * Given a STAR model with constant variance this procedure construct the *
 * likelihoodfunction. Used in Powell's algorithm.                        *
 *                                                                        *
 * in        A row vector that contains all parameter-values required to  *
 *           identify the STAR model. Gamma, Constant(s), the linear AR   *
 *           polynom (incl. the constant) and the non-linear AR polynom.  *
 * ARL       The linear AR part of the STAR model.                        *
 * orgARL    Used to speed up calculations.                               *
 * ARN       The non-linear AR part of the STAR model.                    *
 * Dummys    Dummyvariables used if data is daily.                        *
 * Y, Ydelay Observation vectors.                                         *
 * dstd      Standard deviation of the delay vector.                      *
 * k         Order of the STAR model                                      *
 * restr     The restriction vector.                                      *
 * plin      Apriori for linear AR polynomia.                             *
 *                                                                        *
 * The procedure returns the value of the likelihoodfunction for a given  *
 * parameter vector.                                                      *
 *************************************************************************/

PROC(1)=likstar(in);
LOCAL u,likeli,temp,i,maxab,g,u2,alfa,beta,ARNF;
LOCAL sizeD,sizeL,sizeN,const,sizeA,sizeB,presamp;

sizeD=insize[1];                   /* Identify size of model parameters */
sizeL=insize[2];
sizeN=insize[3];
sizeA=insize[4];
sizeB=insize[5];
const=in[sizeD+sizeL+sizeN+1+k+1];  /* Relevant model parameters */
 alfa=in[sizeD+sizeL+sizeN+1+k+2:sizeD+sizeL+sizeN+1+k+1+sizeA];
 beta=in[sizeD+sizeL+sizeN+1+k+1+sizeA+1:sizeD+sizeL+sizeN+1+k+1+sizeA+sizeB];

{u,ARNF}=resSTAR(in);  /* Calculate residuals for the cond. mean */
presamp=u'u/rows(u);   /* Pre-sample values for u^2 and g */

maxab=maxc(sizeA|sizeB);

u2=ones(rows(u)+maxab,1)*presamp;
 g=ones(rows(u)+maxab,1)*presamp;

i=maxab+1;                     /* index for the first observation, i.e. t>0 */
do until i > rows(u)+maxab;
     g[i]=const; 
    u2[i]=u[i-maxab]^2;
    j=1;
    do until j > maxab;        
        if j <=sizeA;			/* Alfa case */
            g[i]=g[i]+alfa[j]*u2[i-j];
        endif;
        if j <=sizeB;           /* Beta case */
            g[i]=g[i]+beta[j]*g[i-j];
        endif;       
        j=j+1;
    endo;
    i=i+1; 
endo;

g=trimr(g,maxab,0);
u2=trimr(u2,maxab,0);

likeli=-0.5*ln(2*pi)-0.5*ln(g)-0.5*u2./g;

retp(-sumc(likeli));             /* We want to minimize instead of maximize */

ENDP; /********************* END OF PROC LIKSTAR **************************/


/**************************************************************************
 * Procedure resSTAR                                                      *
 *                                                                        *
 * Given a STAR model with constant variance this procedure construct the *
 * likelihoodfunction. Used in Powell's algorithm.                        *
 *                                                                        *
 * in        A row vector that contains all parameter-values required to  *
 *           identify the STAR model. Gamma, Constant(s), the linear AR   *
 *           polynom (incl. the constant) and the non-linear AR polynom.  *
 * ARL       The linear AR part of the STAR model.                        *
 * orgARL    Used to speed up calculations.                               *
 * ARN       The non-linear AR part of the STAR model.                    *
 * Dummys    Dummyvariables used if data is daily.                        *
 * Y, Ydelay Observation vectors.                                         *
 * dstd      Standard deviation of the delay vector.                      *
 * k         Order of the STAR model                                      *
 * restr     The restriction vector.                                      *
 * plin      Apriori for linear AR polynomia.                             *
 *                                                                        *
 * The procedure returns the value of the likelihoodfunction for a given  *
 * parameter vector.                                                      *
 *************************************************************************/

PROC(2)=resSTAR(in);
LOCAL temp,j,prod,F,indrest,STAR,residual,betaD,betaL,betaN,betaDL;
LOCAL g,c,sizeD,sizeL,sizeN,var,likeli;

sizeD=insize[1];
sizeL=insize[2];
sizeN=insize[3];

g=in[sizeD+sizeL+sizeN+1];
c=in[sizeD+sizeL+sizeN+2:sizeD+sizeL+sizeN+1+k];

betaN=in[sizeD+sizeL+1:sizeD+sizeL+sizeN];                         

/* Calculate the non-linear part of the STAR model */

temp=(Ydelay-c[1]);
j=2;
do until j > k;
    temp=temp~(Ydelay-c[j]);
    j=j+1;
endo;

prod=prodc(temp');       /* We need the product of the rows */
prod=prod/dstd^k;        /* Making gamma scale invariant */

F=1/(1+EXP(-g*prod));

/* Nonpart=zeros(rows(ARN),cols(ARN));  i huvudrutin for hastighet */ 

Nonpart=ARN.*F;

IF scalmiss(restr)==0;           /* If true, there is/are restriction(s). */
    indrest=indnv(restr,pnonlin);/* Create a vectorindex for the parameter(s)*/
				                 /* that are restricted in pnonlin. */
    j=1;
    do until j > cols(ARN);
	if scalmiss(indexcat(indrest,j))==0; /* If true, index in ARN is restricted. */
	    Nonpart[.,j]=ARN[.,j].*(F-1);        
	endif;
	j=j+1;
    endo;
ENDIF;

IF scalmiss(ARL)==1 and scalmiss(Dummys)==1;    
    STAR=Nonpart*betaN;    
ELSE;
    betaDL=in[1:sizeD+sizeL];
    STAR=ARDL*betaDL+Nonpart*betaN;            
ENDIF;

residual=Y-STAR;

retp(residual,Nonpart);

ENDP; /********************* END OF PROC resSTAR **************************/


/**************************************************************************/
/**************************** PROC Getgz ********************************/
/**************************************************************************/
                  /*----------- Calculate g and z -----------*/
PROC(2)=getgz(u,const,alfa,beta);
LOCAL p,q,maxpq,u2,g,z,i,j,sizee;

q=rows(alfa);
p=rows(beta);
maxpq=maxc(q|p);

sizee=1+q+p;          /* number of elements in eta vector */  


u2=u.^2;

 g=ones(maxpq,1)*meanc(u2)|zeros(rows(u),1); /* append pre-sample to top of vector */
u2=ones(maxpq,1)*meanc(u2)|u2;
 z=ones(rows(u),sizee);


i=maxpq+1;  /* index for the first observation, i.e. t>0 */
do until i > rows(u)+maxpq;
    g[i]=const; 
    j=1;
    do until j > maxpq;        
        if j <=q;
            g[i]=g[i]+alfa[j]*u2[i-j];
            z[i-maxpq,1+j]=u2[i-j];
        endif;
        if j <=p;
            g[i]=g[i]+beta[j]*g[i-j];
            z[i-maxpq,1+q+j]=g[i-j];
        endif;       
        j=j+1;
    endo;
    i=i+1; 
endo;

g=trimr(g,maxpq,0);  /* remove pre-sample from top of vector */

RETP(g,z);
ENDP;   /************************ END OF PROC getgz ***********************/



